@@ -21,6 +21,7 @@ package org.apache.samza.system.azureblob.avro;
 
 import java.util.Arrays;
 import java.util.HashMap;
+import java.util.concurrent.CountDownLatch;
 import org.apache.samza.AzureException;
 import org.apache.samza.system.azureblob.compression.Compression;
 import org.apache.samza.system.azureblob.producer.AzureBlobWriterMetrics;
@@ -46,6 +47,8 @@ import org.mockito.ArgumentCaptor;
 import org.powermock.api.mockito.PowerMockito;
 import org.powermock.core.classloader.annotations.PrepareForTest;
 import org.powermock.modules.junit4.PowerMockRunner;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 import reactor.core.publisher.Mono;
 
 import static org.mockito.Mockito.any;
@@ -85,6 +88,9 @@ public class TestAzureBlobOutputStream {
   private static final String BLOB_RECORD_NUMBER_METADATA = "numberOfRecords";
   private final BlobMetadataGeneratorFactory blobMetadataGeneratorFactory = mock(BlobMetadataGeneratorFactory.class);
   private final Config blobMetadataGeneratorConfig = mock(Config.class);
+  private final BlobMetadataGenerator mockBlobMetadataGenerator = mock(BlobMetadataGenerator.class);
+
+  private static final Logger LOG = LoggerFactory.getLogger(TestAzureBlobOutputStream.class);
 
   @Before
   public void setup() throws Exception {
@@ -103,7 +109,6 @@ public class TestAzureBlobOutputStream {
     mockCompression = mock(Compression.class);
     doReturn(COMPRESSED_BYTES).when(mockCompression).compress(BYTES);
 
-    BlobMetadataGenerator mockBlobMetadataGenerator = mock(BlobMetadataGenerator.class);
     doAnswer(invocation -> {
       BlobMetadataContext blobMetadataContext = invocation.getArgumentAt(0, BlobMetadataContext.class);
       String streamName = blobMetadataContext.getStreamName();
@@ -127,7 +132,7 @@ public class TestAzureBlobOutputStream {
   }
 
   @Test
-  public void testWrite() {
+  public void testWrite() throws  InterruptedException {
     byte[] b = new byte[THRESHOLD - 10];
     azureBlobOutputStream.write(b, 0, THRESHOLD - 10);
     verify(azureBlobOutputStream, never()).stageBlock(anyString(), any(ByteBuffer.class), anyInt());
@@ -136,7 +141,7 @@ public class TestAzureBlobOutputStream {
   }
 
   @Test
-  public void testWriteLargerThanThreshold() {
+  public void testWriteLargerThanThreshold() throws  InterruptedException {
     byte[] largeRecord = RANDOM_STRING.substring(0, 2 * THRESHOLD).getBytes();
     byte[] largeRecordFirstHalf = RANDOM_STRING.substring(0, THRESHOLD).getBytes();
     byte[] largeRecordSecondHalf = RANDOM_STRING.substring(THRESHOLD, 2 * THRESHOLD).getBytes();
@@ -165,7 +170,7 @@ public class TestAzureBlobOutputStream {
   }
 
   @Test
-  public void testWriteLargeRecordWithSmallRecordInBuffer() {
+  public void testWriteLargeRecordWithSmallRecordInBuffer() throws InterruptedException {
     byte[] halfBlock = new byte[THRESHOLD / 2];
     byte[] fullBlock = new byte[THRESHOLD];
     byte[] largeRecord = new byte[2 * THRESHOLD];
@@ -229,6 +234,36 @@ public class TestAzureBlobOutputStream {
     azureBlobOutputStream.close();
   }
 
+  @Test(expected = AzureException.class)
+  public void testWriteFailedInterruptedException() throws InterruptedException {
+
+    doThrow(new InterruptedException("Lets interrupt the thread"))
+        .when(azureBlobOutputStream).stageBlock(anyString(), any(ByteBuffer.class), anyInt());
+    byte[] b = new byte[100];
+    doReturn(COMPRESSED_BYTES).when(mockCompression).compress(b);
+
+    try {
+      azureBlobOutputStream.write(b, 0, THRESHOLD); // threshold crossed so stageBlock is scheduled.
+      // azureBlobOutputStream.close waits on the CompletableFuture which does the actual stageBlock in uploadBlockAsync
+      azureBlobOutputStream.close();
+    } catch (AzureException exception) {
+      // get root cause of the exception - to confirm its an InterruptedException
+      Throwable dupException = exception;
+      while (dupException.getCause() != null && dupException.getCause() != dupException) {
+        dupException = dupException.getCause();
+      }
+
+      Assert.assertTrue(dupException.getClass().getName().equals(InterruptedException.class.getCanonicalName()));
+      Assert.assertEquals("Lets interrupt the thread", dupException.getMessage());
+
+      // verify stageBlock was called exactly once - aka no retries happen when interrupted exception is thrown
+      verify(azureBlobOutputStream).stageBlock(anyString(), any(ByteBuffer.class), anyInt());
+
+      // rethrow the exception so that the test will fail if no exception was thrown in the try block
+      throw exception;
+    }
+  }
+
   @Test
   public void testClose() {
     azureBlobOutputStream.write(BYTES, 0, THRESHOLD);
@@ -278,7 +313,7 @@ public class TestAzureBlobOutputStream {
   }
 
   @Test(expected = AzureException.class)
-  public void testCloseFailed() {
+  public void testCloseFailed() throws InterruptedException {
 
     azureBlobOutputStream = spy(new AzureBlobOutputStream(mockBlobAsyncClient, threadPool, mockMetrics,
         blobMetadataGeneratorFactory, blobMetadataGeneratorConfig, FAKE_STREAM,
@@ -320,7 +355,7 @@ public class TestAzureBlobOutputStream {
   }
 
   @Test (expected = AzureException.class)
-  public void testFlushFailed() throws IOException {
+  public void testFlushFailed() throws IOException, InterruptedException {
     azureBlobOutputStream = spy(new AzureBlobOutputStream(mockBlobAsyncClient, threadPool, mockMetrics,
         blobMetadataGeneratorFactory, blobMetadataGeneratorConfig, FAKE_STREAM,
         60000, THRESHOLD, mockByteArrayOutputStream, mockCompression));
@@ -386,6 +421,68 @@ public class TestAzureBlobOutputStream {
     Assert.assertEquals(BYTES.length + BYTES.length - 10, azureBlobOutputStream.getSize());
   }
 
+  /**
+   * Test to ensure that flush timeout is respected even if the block upload to azure is stuck/ taking longer than flush timeout
+   * a countdown latch is used to mimic the upload to azure stuck
+   * if flush timeout is respected then an exception is thrown when the flushtimeout_ms duration expires
+   * else if timeout is not respected (aka bug is not fixed) then no exception is thrown and test hangs
+   * In this test, the flush timeout is chosen to be 10 milliseconds, at the end of which, an AzureException of upload failed is thrown.
+   * @throws Exception
+   * @throws InterruptedException
+   */
+  @Test(expected = AzureException.class)
+  public void  testRespectFlushTimeout() throws Exception, InterruptedException {
+    // get the threadpool to be the exactly the same as that passed down to AzureBlobOutputStream from AzureBlobSystemProducer
+    threadPool = new ThreadPoolExecutor(1, 1, 60,
+        TimeUnit.SECONDS, new LinkedBlockingDeque<Runnable>(1), new ThreadPoolExecutor.CallerRunsPolicy());
+
+    // set a very small flushtimeout of 10ms to avoid test taking too long to complete
+    azureBlobOutputStream = spy(new AzureBlobOutputStream(mockBlobAsyncClient, threadPool, mockMetrics,
+        blobMetadataGeneratorFactory, blobMetadataGeneratorConfig, FAKE_STREAM,
+        10, THRESHOLD, mockByteArrayOutputStream, mockCompression));
+
+    doNothing().when(azureBlobOutputStream).clearAndMarkClosed();
+    doReturn(mockBlobMetadataGenerator).when(azureBlobOutputStream).getBlobMetadataGenerator();
+    when(mockCompression.compress(BYTES)).thenReturn(COMPRESSED_BYTES, COMPRESSED_BYTES, COMPRESSED_BYTES, COMPRESSED_BYTES);
+
+    // create a latch to mimic uploads getting stuck
+    // and hence unable to honor flush timeout without the fix in stageblock
+    // fix in stageBlock = subscribeOn(Schedulers.boundedElastic()).block(flushtimeout)
+    CountDownLatch latch = new CountDownLatch(1);
+    doAnswer(invocation -> {
+
+      String blockid = invocation.getArgumentAt(0, String.class);
+      return Mono.just(1).map(integer -> {
+        try {
+          LOG.info("For block id = " + blockid + " start waiting on the countdown latch ");
+          // start indefinite stuck -> mimic upload stuck
+          latch.await();
+          // below log will never be reached
+          LOG.info("For block id = " + blockid + " done waiting on the countdown latch ");
+        } catch (Exception e) {
+          LOG.info("For block id = " + blockid + " an exception was caught " + e);
+        }
+        return "One";
+      });
+    }).when(azureBlobOutputStream).invokeBlobClientStageBlock(anyString(), anyObject(), anyInt());
+
+    doAnswer(invocation -> {
+      LOG.info("commit block ");
+      return null;
+    }).when(azureBlobOutputStream).commitBlob(anyObject(), anyMap());
+
+    azureBlobOutputStream.write(BYTES, 0, THRESHOLD / 2);
+    azureBlobOutputStream.write(BYTES, THRESHOLD / 2, THRESHOLD / 2);
+    azureBlobOutputStream.write(BYTES, 0, THRESHOLD / 2);
+    azureBlobOutputStream.write(BYTES, THRESHOLD / 2, THRESHOLD / 2);
+    azureBlobOutputStream.write(BYTES, 0, THRESHOLD / 2);
+    azureBlobOutputStream.write(BYTES, THRESHOLD / 2, THRESHOLD / 2);
+    // close will wait for all pending uploads to finish
+    // since the uploads are "stuck" (waiting for latch countdown), flushtimeout will get triggered
+    // and throw an exception saying upload failed.
+    azureBlobOutputStream.close();
+  }
+
   private String blockIdEncoded(int blockNum) {
     String blockId = String.format("%05d", blockNum);
     return Base64.getEncoder().encodeToString(blockId.getBytes());
