@@ -24,12 +24,15 @@ import org.apache.samza.config.ApplicationConfig;
 import org.apache.samza.config.Config;
 import org.apache.samza.config.ConfigException;
 import org.apache.samza.config.TaskConfig;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 
 /**
  * Util class to create {@link SamzaApplication} from the configuration.
  */
 public class ApplicationUtil {
+  private static final Logger LOG = LoggerFactory.getLogger(ApplicationUtil.class);
 
   /**
    * Creates the {@link SamzaApplication} object from the task or application class name specified in {@code config}
@@ -38,6 +41,7 @@ public class ApplicationUtil {
    * @return the {@link SamzaApplication} object
    */
   public static SamzaApplication fromConfig(Config config) {
+
     String appClassName = new ApplicationConfig(config).getAppClass();
     if (StringUtils.isNotBlank(appClassName)) {
       // app.class is configured
@@ -60,4 +64,13 @@ public class ApplicationUtil {
     }
     return new LegacyTaskApplication(taskClassOption.get());
   }
-}
\ No newline at end of file
+
+  /**
+   * Determines if the job is a Samza high-level job.
+   * @param config config
+   * */
+  public static boolean isHighLevelApiJob(Config config) {
+    final ApplicationConfig applicationConfig = new ApplicationConfig(config);
+    return applicationConfig.getAppApiType() == ApplicationApiType.HIGH_LEVEL;
+  }
+}
