@@ -24,6 +24,7 @@ import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -34,6 +35,7 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.Collectors;
 import org.apache.samza.SamzaException;
+import org.apache.samza.system.DrainMessage;
 import org.apache.samza.system.IncomingMessageEnvelope;
 import org.apache.samza.system.MessageType;
 import org.apache.samza.system.SystemConsumers;
@@ -69,7 +71,6 @@ public class RunLoop implements Runnable, Throttleable {
   private final List<AsyncTaskWorker> taskWorkers;
   private final SystemConsumers consumerMultiplexer;
   private final Map<SystemStreamPartition, List<AsyncTaskWorker>> sspToTaskWorkerMapping;
-
   private final ExecutorService threadPool;
   private final CoordinatorRequests coordinatorRequests;
   private final Object latch;
@@ -77,6 +78,7 @@ public class RunLoop implements Runnable, Throttleable {
   private final long windowMs;
   private final long commitMs;
   private final long callbackTimeoutMs;
+  private final long drainCallbackTimeoutMs;
   private final long maxIdleMs;
   private final SamzaContainerMetrics containerMetrics;
   private final ScheduledExecutorService workerTimer;
@@ -85,9 +87,12 @@ public class RunLoop implements Runnable, Throttleable {
   private volatile boolean shutdownNow = false;
   private volatile Throwable throwable = null;
   private final HighResolutionClock clock;
-  private final boolean isAsyncCommitEnabled;
+  private boolean isAsyncCommitEnabled;
   private volatile boolean runLoopResumedSinceLastChecked;
   private final int elasticityFactor;
+  private final String runId;
+  private final boolean isHighLevelApiJob;
+  private boolean isDraining = false;
 
   public RunLoop(Map<TaskName, RunLoopTask> runLoopTasks,
       ExecutorService threadPool,
@@ -96,13 +101,14 @@ public class RunLoop implements Runnable, Throttleable {
       long windowMs,
       long commitMs,
       long callbackTimeoutMs,
+      long drainCallbackTimeoutMs,
       long maxThrottlingDelayMs,
       long maxIdleMs,
       SamzaContainerMetrics containerMetrics,
       HighResolutionClock clock,
       boolean isAsyncCommitEnabled) {
     this(runLoopTasks, threadPool, consumerMultiplexer, maxConcurrency, windowMs, commitMs, callbackTimeoutMs,
-        maxThrottlingDelayMs, maxIdleMs, containerMetrics, clock, isAsyncCommitEnabled, 1);
+        drainCallbackTimeoutMs, maxThrottlingDelayMs, maxIdleMs, containerMetrics, clock, isAsyncCommitEnabled, 1, null, false);
   }
 
   public RunLoop(Map<TaskName, RunLoopTask> runLoopTasks,
@@ -112,12 +118,15 @@ public class RunLoop implements Runnable, Throttleable {
       long windowMs,
       long commitMs,
       long callbackTimeoutMs,
+      long drainCallbackTimeoutMs,
       long maxThrottlingDelayMs,
       long maxIdleMs,
       SamzaContainerMetrics containerMetrics,
       HighResolutionClock clock,
       boolean isAsyncCommitEnabled,
-      int elasticityFactor) {
+      int elasticityFactor,
+      String runId,
+      boolean isHighLevelApiJob) {
 
     this.threadPool = threadPool;
     this.consumerMultiplexer = consumerMultiplexer;
@@ -126,6 +135,7 @@ public class RunLoop implements Runnable, Throttleable {
     this.commitMs = commitMs;
     this.maxConcurrency = maxConcurrency;
     this.callbackTimeoutMs = callbackTimeoutMs;
+    this.drainCallbackTimeoutMs = drainCallbackTimeoutMs;
     this.maxIdleMs = maxIdleMs;
     this.callbackTimer = (callbackTimeoutMs > 0) ? Executors.newSingleThreadScheduledExecutor() : null;
     this.callbackExecutor = new ThrottlingScheduler(maxThrottlingDelayMs);
@@ -133,15 +143,30 @@ public class RunLoop implements Runnable, Throttleable {
     this.latch = new Object();
     this.workerTimer = Executors.newSingleThreadScheduledExecutor();
     this.clock = clock;
-    Map<TaskName, AsyncTaskWorker> workers = new HashMap<>();
+    // assign runId before creating workers. As the inner AsyncTaskWorker class is not static, it relies on
+    // the outer class fields to be init first
+    this.runId = runId;
+    this.isHighLevelApiJob = isHighLevelApiJob;
+    this.isAsyncCommitEnabled = isAsyncCommitEnabled;
+    this.elasticityFactor = elasticityFactor;
+
+    Map<TaskName, AsyncTaskWorker>  workers = new HashMap<>();
     for (RunLoopTask task : runLoopTasks.values()) {
       workers.put(task.taskName(), new AsyncTaskWorker(task));
     }
     // Partions and tasks assigned to the container will not change during the run loop life time
     this.sspToTaskWorkerMapping = Collections.unmodifiableMap(getSspToAsyncTaskWorkerMap(runLoopTasks, workers));
     this.taskWorkers = Collections.unmodifiableList(new ArrayList<>(workers.values()));
-    this.isAsyncCommitEnabled = isAsyncCommitEnabled;
-    this.elasticityFactor = elasticityFactor;
+  }
+
+  /**
+   * Sets the RunLoop to drain mode.
+   * */
+  private void drain() {
+    log.info("Setting the RunLoop to drain mode.");
+    isDraining = true;
+    log.debug("Disabling async commit when the pipeline is draining.");
+    isAsyncCommitEnabled = false;
   }
 
   /**
@@ -270,12 +295,18 @@ public class RunLoop implements Runnable, Throttleable {
           }
         } else if (elasticityFactor > 1) {
           // is listOfWorkersForEnvelope is null and elascity factor > 1 (aka enabled), then
-          // this condition happens when a keyBucket of the SSP is being consumed but other keyBuckets are not consumed
+          // this condition happens when only a subset of keyBuckets of an SSP are being consumed at this container
+          // but not all keyBuckets of an SSP are consumed, therefore, we fake the consumption of the envelope,
+          // and decrement the metric, since this envelope's processing is skipped.
           // if this update is not done for the SSP then the unprocessed envelopes from other keyBuckets
-          // will make the consumerMultiplexer not poll as it sees envelopes available for consumption.
+          // will prevent the consumerMultiplexer from polling as it sees envelopes available for consumption.
           consumerMultiplexer.tryUpdate(envelope.getSystemStreamPartition(elasticityFactor));
           log.trace("updating the system consumers for ssp keyBucket {} not processed by this runloop",
               envelope.getSystemStreamPartition(elasticityFactor));
+          // since this envelope is not processed by the container, need to decrement the # envelopes metric
+          // # envelopes metric was incremented when the envelope was returned by the SystemConsumers
+          containerMetrics.envelopes().dec();
+          containerMetrics.skippedEnvelopes().inc();
         }
       }
 
@@ -290,7 +321,8 @@ public class RunLoop implements Runnable, Throttleable {
    * when elasticity is enabled,
    *       sspToTaskWorkerMapping has workers for a SSP which has keyBucket
    *       hence need to use envelop.getSSP(elasticityFactor)
-   *       Additionally, when envelope is EnofStream or Watermark, it needs to be sent to all works for the ssp irrespective of keyBucket
+   *       Additionally, when envelope is EndOfStream or Watermark or Drain, it needs to be sent to all works for the ssp
+   *       irrespective of keyBucket
    * @param envelope
    * @return list of workers for the envelope
    */
@@ -302,9 +334,14 @@ public class RunLoop implements Runnable, Throttleable {
     final SystemStreamPartition sspOfEnvelope = envelope.getSystemStreamPartition(elasticityFactor);
     List<AsyncTaskWorker> listOfWorkersForEnvelope = null;
 
-    // if envelope is end of stream or watermark, it needs to be routed to all tasks consuming the ssp irresp of keybucket
+    // if envelope is end of stream or watermark or drain, it needs to be routed to all tasks consuming the ssp irresp
+    // of keybucket
     MessageType messageType = MessageType.of(envelope.getMessage());
-    if (envelope.isEndOfStream() || MessageType.END_OF_STREAM == messageType || MessageType.WATERMARK == messageType) {
+    if (envelope.isEndOfStream()
+        || envelope.isDrain()
+        || messageType == MessageType.END_OF_STREAM
+        || messageType == MessageType.DRAIN
+        || messageType == MessageType.WATERMARK) {
 
       //sspToTaskWorkerMapping has ssps with keybucket so extract and check only system, stream and partition and ignore the keybucket
       listOfWorkersForEnvelope = sspToTaskWorkerMapping.entrySet()
@@ -383,6 +420,20 @@ public class RunLoop implements Runnable, Throttleable {
     }
   }
 
+  /**
+   * Resume the runloop thread. This method is triggered after a task has completed drain.
+   */
+  private void resumeAfterDrain() {
+    log.trace("Resume loop thread");
+    if (coordinatorRequests.shouldShutdownNow()) {
+      shutdownNow = true;
+    }
+    synchronized (latch) {
+      latch.notifyAll();
+      runLoopResumedSinceLastChecked = true;
+    }
+  }
+
   /**
    * Set the throwable and abort run loop. The throwable will be thrown from the run loop thread
    * @param t throwable
@@ -419,6 +470,7 @@ public class RunLoop implements Runnable, Throttleable {
     COMMIT,
     PROCESS,
     END_OF_STREAM,
+    DRAIN,
     SCHEDULER,
     NO_OP
   }
@@ -436,7 +488,8 @@ public class RunLoop implements Runnable, Throttleable {
       this.task = task;
       this.callbackManager = new TaskCallbackManager(this, callbackTimer, callbackTimeoutMs, maxConcurrency, clock);
       Set<SystemStreamPartition> sspSet = getWorkingSSPSet(task);
-      this.state = new AsyncTaskState(task.taskName(), task.metrics(), sspSet, !task.intermediateStreams().isEmpty());
+      this.state = new AsyncTaskState(task.taskName(), task.metrics(), sspSet, !task.intermediateStreams().isEmpty(), isHighLevelApiJob,
+          runId);
     }
 
     private void init() {
@@ -507,18 +560,46 @@ public class RunLoop implements Runnable, Throttleable {
         case END_OF_STREAM:
           endOfStream();
           break;
+        case DRAIN:
+          drain();
+          break;
         default:
           //no op
           break;
       }
     }
 
+    /**
+     * Called when a task has drained i.e all SSPs for the task have received a drain message.
+     * */
+    private void drain() {
+      state.complete = true;
+      state.startDrain();
+      try {
+        ReadableCoordinator coordinator = new ReadableCoordinator(task.taskName());
+
+        task.drain(coordinator);
+
+        // issue a shutdown request for the task
+        coordinator.shutdown(TaskCoordinator.RequestScope.CURRENT_TASK);
+        coordinatorRequests.update(coordinator);
+
+        // issue a commit explicitly before we shutdown the task
+        // Adding commit to coordinator will not work as the state is marked complete and NO_OP will always be the
+        // next operation for this task
+        commit();
+      } finally {
+        resumeAfterDrain();
+      }
+    }
+
     private void endOfStream() {
       state.complete = true;
       try {
         ReadableCoordinator coordinator = new ReadableCoordinator(task.taskName());
 
         task.endOfStream(coordinator);
+
         // issue a request for shutdown of the task
         coordinator.shutdown(TaskCoordinator.RequestScope.CURRENT_TASK);
         coordinatorRequests.update(coordinator);
@@ -531,7 +612,6 @@ public class RunLoop implements Runnable, Throttleable {
       } finally {
         resume();
       }
-
     }
 
     /**
@@ -547,7 +627,9 @@ public class RunLoop implements Runnable, Throttleable {
         public TaskCallback createCallback() {
           state.startProcess();
           containerMetrics.processes().inc();
-          return callbackManager.createCallback(task.taskName(), envelope, coordinator);
+          return isDraining && (envelope.isDrain() || envelope.isWatermark())
+              ? callbackManager.createCallbackForDrain(task.taskName(), envelope, coordinator, drainCallbackTimeoutMs)
+              : callbackManager.createCallback(task.taskName(), envelope, coordinator);
         }
       };
 
@@ -631,9 +713,12 @@ public class RunLoop implements Runnable, Throttleable {
         }
       };
 
-      if (threadPool != null) {
+      if (threadPool != null && !isDraining) {
         log.trace("Task {} commits on the thread pool", task.taskName());
         threadPool.submit(commitWorker);
+      } else if (isDraining) {
+        log.trace("Task {} commits on the run loop thread as task is draining", task.taskName());
+        commitWorker.run();
       } else {
         log.trace("Task {} commits on the run loop thread", task.taskName());
         commitWorker.run();
@@ -752,24 +837,33 @@ public class RunLoop implements Runnable, Throttleable {
     private volatile boolean needScheduler = false;
     private volatile boolean complete = false;
     private volatile boolean endOfStream = false;
+    private volatile boolean shouldDrain = false;
     private volatile boolean windowInFlight = false;
     private volatile boolean commitInFlight = false;
     private volatile boolean schedulerInFlight = false;
     private final AtomicInteger messagesInFlight = new AtomicInteger(0);
-    private final ArrayDeque<PendingEnvelope> pendingEnvelopeQueue;
+    private final ArrayDeque<PendingEnvelope>   pendingEnvelopeQueue;
 
     //Set of SSPs that we are currently processing for this task instance
     private final Set<SystemStreamPartition> processingSspSet;
+    //Set of SSPs that we are currently processing for this task instance
+    private final Set<SystemStreamPartition> processingSspSetToDrain;
     private final TaskName taskName;
     private final TaskInstanceMetrics taskMetrics;
     private final boolean hasIntermediateStreams;
+    private final boolean isHighLevelApiJob;
+    private final String runId;
 
-    AsyncTaskState(TaskName taskName, TaskInstanceMetrics taskMetrics, Set<SystemStreamPartition> sspSet, boolean hasIntermediateStreams) {
+    AsyncTaskState(TaskName taskName, TaskInstanceMetrics taskMetrics, Set<SystemStreamPartition> sspSet,
+        boolean hasIntermediateStreams, boolean isHighLevelApiJob, String runId) {
       this.taskName = taskName;
       this.taskMetrics = taskMetrics;
       this.pendingEnvelopeQueue = new ArrayDeque<>();
       this.processingSspSet = sspSet;
+      this.processingSspSetToDrain = new HashSet<>(sspSet);
       this.hasIntermediateStreams = hasIntermediateStreams;
+      this.isHighLevelApiJob = isHighLevelApiJob;
+      this.runId = runId;
     }
 
     private boolean checkEndOfStream() {
@@ -788,6 +882,46 @@ public class RunLoop implements Runnable, Throttleable {
       return processingSspSet.isEmpty();
     }
 
+    private boolean shouldDrain() {
+      if (endOfStream) {
+        return false;
+      }
+
+      if (pendingEnvelopeQueue.size() > 0) {
+        final PendingEnvelope pendingEnvelope = pendingEnvelopeQueue.peek();
+        final IncomingMessageEnvelope envelope = pendingEnvelope.envelope;
+
+        if (envelope.isDrain()) {
+          final DrainMessage message = (DrainMessage) envelope.getMessage();
+          if (!message.getRunId().equals(runId)) {
+            // Removing the drain message from the pending queue as it doesn't match with the current deployment
+            final PendingEnvelope discardedDrainMessage = pendingEnvelopeQueue.remove();
+            consumerMultiplexer.tryUpdate(discardedDrainMessage.envelope.getSystemStreamPartition());
+          } else {
+            // Found drain message matching the current deployment
+
+            // set the RunLoop to drain mode
+            if (!isDraining) {
+              drain();
+            }
+
+            if (!isHighLevelApiJob) {
+              // The flow below only applies to samza low-level API
+
+              // For high-level API, we do not remove the message from pending queue.
+              // It will be picked by the process flow instead of drain flow, as we want the drain control message
+              // to be processed by the High-level API Operator DAG.
+
+              processingSspSetToDrain.remove(envelope.getSystemStreamPartition());
+              pendingEnvelopeQueue.remove();
+            }
+          }
+        }
+      }
+
+      return processingSspSetToDrain.isEmpty();
+    }
+
     /**
      * Returns whether the task is ready to do process/window/commit.
      *
@@ -796,6 +930,11 @@ public class RunLoop implements Runnable, Throttleable {
       if (checkEndOfStream()) {
         endOfStream = true;
       }
+
+      if (shouldDrain()) {
+        shouldDrain = true;
+      }
+
       if (coordinatorRequests.commitRequests().remove(taskName)) {
         needCommit = true;
       }
@@ -809,9 +948,9 @@ public class RunLoop implements Runnable, Throttleable {
        */
       if (needCommit) {
         return (messagesInFlight.get() == 0 || isAsyncCommitEnabled) && !opInFlight;
-      } else if (needWindow || needScheduler || endOfStream) {
+      } else if (needWindow || needScheduler || endOfStream || shouldDrain) {
         /*
-         * A task is ready for window, scheduler or end-of-stream operation.
+         * A task is ready for window, scheduler, drain or end-of-stream operation.
          */
         return messagesInFlight.get() == 0 && !opInFlight;
       } else {
@@ -830,14 +969,24 @@ public class RunLoop implements Runnable, Throttleable {
      */
     private WorkerOp nextOp() {
 
-      if (complete) return WorkerOp.NO_OP;
+      if (complete) {
+        return WorkerOp.NO_OP;
+      }
 
       if (isReady()) {
-        if (needCommit) return WorkerOp.COMMIT;
-        else if (needWindow) return WorkerOp.WINDOW;
-        else if (needScheduler) return WorkerOp.SCHEDULER;
-        else if (endOfStream && pendingEnvelopeQueue.isEmpty()) return WorkerOp.END_OF_STREAM;
-        else if (!pendingEnvelopeQueue.isEmpty()) return WorkerOp.PROCESS;
+        if (needCommit) {
+          return WorkerOp.COMMIT;
+        } else if (needWindow) {
+          return WorkerOp.WINDOW;
+        } else if (needScheduler) {
+          return WorkerOp.SCHEDULER;
+        } else if (endOfStream && pendingEnvelopeQueue.isEmpty()) {
+          return WorkerOp.END_OF_STREAM;
+        } else if (shouldDrain && pendingEnvelopeQueue.isEmpty()) {
+          return WorkerOp.DRAIN;
+        } else if (!pendingEnvelopeQueue.isEmpty()) {
+          return WorkerOp.PROCESS;
+        }
       }
       return WorkerOp.NO_OP;
     }
@@ -859,6 +1008,10 @@ public class RunLoop implements Runnable, Throttleable {
       windowInFlight = true;
     }
 
+    private void startDrain() {
+      shouldDrain = false;
+    }
+
     private void startCommit() {
       needCommit = false;
       commitInFlight = true;
@@ -901,6 +1054,7 @@ public class RunLoop implements Runnable, Throttleable {
       int queueSize = pendingEnvelopeQueue.size();
       taskMetrics.pendingMessages().set(queueSize);
       log.trace("Insert envelope to task {} queue.", taskName);
+      log.trace("Insert envelope to task {} queue.", taskName);
       log.debug("Task {} pending envelope count is {} after insertion.", taskName, queueSize);
     }
 
