@@ -41,7 +41,9 @@ public class RunLoopFactory {
       SamzaContainerMetrics containerMetrics,
       TaskConfig taskConfig,
       HighResolutionClock clock,
-      int elasticityFactor) {
+      int elasticityFactor,
+      String runId,
+      boolean isHighLevelApiJob) {
 
     long taskWindowMs = taskConfig.getWindowMs();
 
@@ -60,11 +62,22 @@ public class RunLoopFactory {
     long callbackTimeout = taskConfig.getCallbackTimeoutMs();
     log.info("Got callbackTimeout: {}.", callbackTimeout);
 
+    long drainCallbackTimeout = taskConfig.getDrainCallbackTimeoutMs();
+    log.info("Got callback timeout for drain: {}.", callbackTimeout);
+
     long maxIdleMs = taskConfig.getMaxIdleMs();
     log.info("Got maxIdleMs: {}.", maxIdleMs);
 
     log.info("Got elasticity factor: {}.", elasticityFactor);
 
+    log.info("Got current run Id: {}.", runId);
+
+    if (isHighLevelApiJob) {
+      log.info("The application uses high-level API.");
+    } else {
+      log.info("The application doesn't use high-level API.");
+    }
+
     log.info("Run loop in asynchronous mode.");
 
     return new RunLoop(
@@ -75,11 +88,14 @@ public class RunLoopFactory {
       taskWindowMs,
       taskCommitMs,
       callbackTimeout,
+      drainCallbackTimeout,
       maxThrottlingDelayMs,
       maxIdleMs,
       containerMetrics,
       clock,
       isAsyncCommitEnabled,
-      elasticityFactor);
+      elasticityFactor,
+      runId,
+      isHighLevelApiJob);
   }
 }
