@@ -29,7 +29,7 @@ import org.apache.samza.SamzaException;
 import org.apache.samza.config.Config;
 import org.apache.samza.config.ConfigException;
 import org.apache.samza.config.JobConfig;
-import org.apache.samza.config.TaskConfigJava;
+import org.apache.samza.config.TaskConfig;
 import org.apache.samza.container.TaskName;
 import org.apache.samza.system.SystemStreamPartition;
 
@@ -61,10 +61,10 @@ class AllSspToSingleTaskGrouper implements SystemStreamPartitionGrouper {
     }
 
     processorList.forEach(processor -> {
-        // Create a task name for each processor and assign all partitions to each task name.
-        final TaskName taskName = new TaskName(String.format("Task-%s", processor));
-        groupedMap.put(taskName, ssps);
-      });
+      // Create a task name for each processor and assign all partitions to each task name.
+      final TaskName taskName = new TaskName(String.format("Task-%s", processor));
+      groupedMap.put(taskName, ssps);
+    });
 
     return groupedMap;
   }
@@ -73,11 +73,11 @@ class AllSspToSingleTaskGrouper implements SystemStreamPartitionGrouper {
 public class AllSspToSingleTaskGrouperFactory implements SystemStreamPartitionGrouperFactory {
   @Override
   public SystemStreamPartitionGrouper getSystemStreamPartitionGrouper(Config config) {
-    if (!(new TaskConfigJava(config).getBroadcastSystemStreams().isEmpty())) {
+    if (!(new TaskConfig(config).getBroadcastSystemStreams().isEmpty())) {
       throw new ConfigException("The job configured with AllSspToSingleTaskGrouper cannot have broadcast streams.");
     }
 
-    String processors = config.get(JobConfig.PROCESSOR_LIST());
+    String processors = config.get(JobConfig.PROCESSOR_LIST);
     List<String> processorList = Arrays.asList(processors.split(","));
     if (processorList.isEmpty()) {
       throw new SamzaException("processor list cannot be empty!");
