@@ -70,8 +70,8 @@ public class CoordinatorStreamSystemConsumer {
   public CoordinatorStreamSystemConsumer(Config config, MetricsRegistry registry) {
     SystemStream coordinatorSystemStream = CoordinatorStreamUtil.getCoordinatorSystemStream(config);
     SystemFactory systemFactory = CoordinatorStreamUtil.getCoordinatorSystemFactory(config);
-    SystemAdmin systemAdmin = systemFactory.getAdmin(coordinatorSystemStream.getSystem(), config);
-    SystemConsumer systemConsumer = systemFactory.getConsumer(coordinatorSystemStream.getSystem(), config, registry);
+    SystemAdmin systemAdmin = systemFactory.getAdmin(coordinatorSystemStream.getSystem(), config, this.getClass().getSimpleName());
+    SystemConsumer systemConsumer = systemFactory.getConsumer(coordinatorSystemStream.getSystem(), config, registry, this.getClass().getSimpleName());
 
     this.coordinatorSystemStreamPartition = new SystemStreamPartition(coordinatorSystemStream, new Partition(0));
     this.systemConsumer = systemConsumer;
@@ -202,13 +202,11 @@ public class CoordinatorStreamSystemConsumer {
     }
   }
 
-  public Set<CoordinatorStreamMessage> getBoostrappedStream() {
-    log.info("Returning the bootstrapped data from the stream");
-    if (!isBootstrapped)
-      bootstrap();
-    return bootstrappedStreamSet;
-  }
-
+  /**
+   * Returns the set of bootstrapped {@link CoordinatorStreamMessage}s
+   * @param type The type of {@link CoordinatorStreamMessage}s to return.
+   * @return The bootstrapped {@link CoordinatorStreamMessage}s
+   */
   public Set<CoordinatorStreamMessage> getBootstrappedStream(String type) {
     log.debug("Bootstrapping coordinator stream for messages of type {}", type);
     bootstrap();
