@@ -24,31 +24,43 @@ import com.google.common.base.Preconditions;
 import com.google.common.collect.ArrayListMultimap;
 import com.google.common.collect.ImmutableMap;
 import com.google.common.collect.ListMultimap;
-
 import java.io.File;
 import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
+import java.util.Set;
+import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.ExecutorService;
 import org.apache.commons.lang3.StringUtils;
 import org.apache.samza.Partition;
 import org.apache.samza.SamzaException;
-import org.apache.samza.checkpoint.CheckpointedChangelogOffset;
+import org.apache.samza.checkpoint.Checkpoint;
+import org.apache.samza.checkpoint.CheckpointId;
+import org.apache.samza.checkpoint.CheckpointV1;
+import org.apache.samza.checkpoint.CheckpointV2;
+import org.apache.samza.checkpoint.kafka.KafkaChangelogSSPOffset;
+import org.apache.samza.checkpoint.kafka.KafkaStateCheckpointMarker;
 import org.apache.samza.config.Config;
 import org.apache.samza.config.StorageConfig;
 import org.apache.samza.config.TaskConfig;
 import org.apache.samza.container.TaskName;
+import org.apache.samza.context.ContainerContext;
+import org.apache.samza.context.JobContext;
 import org.apache.samza.job.model.TaskMode;
 import org.apache.samza.job.model.TaskModel;
+import org.apache.samza.metrics.MetricsRegistry;
+import org.apache.samza.serializers.Serde;
 import org.apache.samza.system.ChangelogSSPIterator;
 import org.apache.samza.system.SSPMetadataCache;
 import org.apache.samza.system.SystemAdmin;
 import org.apache.samza.system.SystemAdmins;
 import org.apache.samza.system.SystemConsumer;
 import org.apache.samza.system.SystemStream;
-import org.apache.samza.system.SystemStreamPartition;
 import org.apache.samza.system.SystemStreamMetadata.SystemStreamPartitionMetadata;
+import org.apache.samza.system.SystemStreamPartition;
+import org.apache.samza.task.MessageCollector;
 import org.apache.samza.util.Clock;
 import org.apache.samza.util.FileUtil;
 import org.slf4j.Logger;
@@ -64,7 +76,7 @@ public class TransactionalStateTaskRestoreManager implements TaskRestoreManager
   private final Map<String, StorageEngine> storeEngines; // store name to storage engines
   private final Map<String, SystemStream> storeChangelogs; // store name to changelog system stream
   private final SystemAdmins systemAdmins;
-  private final Map<String, SystemConsumer> storeConsumers;
+  private final Map<String, SystemConsumer> storeConsumers; // store name to system consumer
   private final SSPMetadataCache sspMetadataCache;
   private final File loggedStoreBaseDirectory;
   private final File nonLoggedStoreBaseDirectory;
@@ -72,23 +84,32 @@ public class TransactionalStateTaskRestoreManager implements TaskRestoreManager
   private final Clock clock;
   private final StorageManagerUtil storageManagerUtil;
   private final FileUtil fileUtil;
+  private final ExecutorService restoreExecutor;
 
   private StoreActions storeActions; // available after init
   private Map<SystemStreamPartition, SystemStreamPartitionMetadata> currentChangelogOffsets;
 
   public TransactionalStateTaskRestoreManager(
+      Set<String> storeNames,  // non-side input stores
+      JobContext jobContext,
+      ContainerContext containerContext,
       TaskModel taskModel,
-      Map<String, StorageEngine> storeEngines,
+      ExecutorService restoreExecutor,
       Map<String, SystemStream> storeChangelogs,
+      Map<String, StorageEngine> inMemoryStores, // in memory stores to be mutated during restore
+      Map<String, StorageEngineFactory<Object, Object>> storageEngineFactories,
+      Map<String, Serde<Object>> serdes,
       SystemAdmins systemAdmins,
       Map<String, SystemConsumer> storeConsumers,
+      MetricsRegistry metricsRegistry,
+      MessageCollector messageCollector,
       SSPMetadataCache sspMetadataCache,
       File loggedStoreBaseDirectory,
       File nonLoggedStoreBaseDirectory,
       Config config,
       Clock clock) {
     this.taskModel = taskModel;
-    this.storeEngines = storeEngines;
+    this.restoreExecutor = restoreExecutor;
     this.storeChangelogs = storeChangelogs;
     this.systemAdmins = systemAdmins;
     this.storeConsumers = storeConsumers;
@@ -101,14 +122,17 @@ public class TransactionalStateTaskRestoreManager implements TaskRestoreManager
     this.clock = clock;
     this.storageManagerUtil = new StorageManagerUtil();
     this.fileUtil = new FileUtil();
+    this.storeEngines = createStoreEngines(storeNames, jobContext, containerContext,
+        storageEngineFactories, serdes, metricsRegistry, messageCollector, inMemoryStores);
   }
 
   @Override
-  public void init(Map<SystemStreamPartition, String> checkpointedChangelogOffsets) {
+  public void init(Checkpoint checkpoint) {
+    Map<String, KafkaStateCheckpointMarker> storeStateCheckpointMarkers = getCheckpointedChangelogOffsets(checkpoint);
     currentChangelogOffsets = getCurrentChangelogOffsets(taskModel, storeChangelogs, sspMetadataCache);
 
     this.storeActions = getStoreActions(taskModel, storeEngines, storeChangelogs,
-        checkpointedChangelogOffsets, currentChangelogOffsets, systemAdmins, storageManagerUtil,
+        storeStateCheckpointMarkers, getCheckpointId(checkpoint), currentChangelogOffsets, systemAdmins, storageManagerUtil,
         loggedStoreBaseDirectory, nonLoggedStoreBaseDirectory, config, clock);
 
     setupStoreDirs(taskModel, storeEngines, storeActions, storageManagerUtil, fileUtil,
@@ -118,32 +142,40 @@ public class TransactionalStateTaskRestoreManager implements TaskRestoreManager
   }
 
   @Override
-  public void restore() throws InterruptedException {
-    Map<String, RestoreOffsets> storesToRestore = storeActions.storesToRestore;
+  public CompletableFuture<Void> restore() {
+    return CompletableFuture.runAsync(() -> {
+      Map<String, RestoreOffsets> storesToRestore = storeActions.storesToRestore;
 
-    for (Map.Entry<String, RestoreOffsets> entry : storesToRestore.entrySet()) {
-      String storeName = entry.getKey();
-      String endOffset = entry.getValue().endingOffset;
-      SystemStream systemStream = storeChangelogs.get(storeName);
-      SystemAdmin systemAdmin = systemAdmins.getSystemAdmin(systemStream.getSystem());
-      SystemConsumer systemConsumer = storeConsumers.get(storeName);
-      SystemStreamPartition changelogSSP = new SystemStreamPartition(systemStream, taskModel.getChangelogPartition());
+      for (Map.Entry<String, RestoreOffsets> entry : storesToRestore.entrySet()) {
+        String storeName = entry.getKey();
+        String endOffset = entry.getValue().endingOffset;
+        SystemStream systemStream = storeChangelogs.get(storeName);
+        SystemAdmin systemAdmin = systemAdmins.getSystemAdmin(systemStream.getSystem());
+        SystemConsumer systemConsumer = storeConsumers.get(storeName);
+        SystemStreamPartition changelogSSP = new SystemStreamPartition(systemStream, taskModel.getChangelogPartition());
 
-      ChangelogSSPIterator changelogSSPIterator =
-          new ChangelogSSPIterator(systemConsumer, changelogSSP, endOffset, systemAdmin, true,
-              currentChangelogOffsets.get(changelogSSP).getNewestOffset());
-      StorageEngine taskStore = storeEngines.get(storeName);
+        ChangelogSSPIterator changelogSSPIterator =
+            new ChangelogSSPIterator(systemConsumer, changelogSSP, endOffset, systemAdmin, true,
+                currentChangelogOffsets.get(changelogSSP).getNewestOffset());
+        StorageEngine taskStore = storeEngines.get(storeName);
 
-      LOG.info("Restoring store: {} for task: {}", storeName, taskModel.getTaskName());
-      taskStore.restore(changelogSSPIterator);
-    }
+        LOG.info("Restoring store: {} for task: {}", storeName, taskModel.getTaskName());
+        try {
+          taskStore.restore(changelogSSPIterator);
+        } catch (InterruptedException e) {
+          String msg = String.format("Interrupted while restoring store: %s for task: %s",
+              storeName, taskModel.getTaskName().getTaskName());
+          throw new SamzaException(msg, e); // wrap in unchecked exception to throw from lambda
+        }
+      }
+    }, restoreExecutor);
   }
 
   /**
    * Stop only persistent stores. In case of certain stores and store mode (such as RocksDB), this
    * can invoke compaction. Persisted stores are recreated in read-write mode in {@link ContainerStorageManager}.
    */
-  public void stopPersistentStores() {
+  public void close() {
     TaskName taskName = taskModel.getTaskName();
     storeEngines.forEach((storeName, storeEngine) -> {
       if (storeEngine.getStoreProperties().isPersistedToDisk())
@@ -152,6 +184,27 @@ public class TransactionalStateTaskRestoreManager implements TaskRestoreManager
     });
   }
 
+  private Map<String, StorageEngine> createStoreEngines(Set<String> storeNames, JobContext jobContext,
+      ContainerContext containerContext, Map<String, StorageEngineFactory<Object, Object>> storageEngineFactories,
+      Map<String, Serde<Object>> serdes, MetricsRegistry metricsRegistry,
+      MessageCollector messageCollector, Map<String, StorageEngine> nonPersistedStores) {
+    Map<String, StorageEngine> storageEngines = new HashMap<>();
+    // Put non persisted stores
+    nonPersistedStores.forEach(storageEngines::put);
+    // Create persisted stores
+    storeNames.forEach(storeName -> {
+      boolean isLogged = this.storeChangelogs.containsKey(storeName);
+      File storeBaseDir = isLogged ? this.loggedStoreBaseDirectory : this.nonLoggedStoreBaseDirectory;
+      File storeDirectory = storageManagerUtil.getTaskStoreDir(storeBaseDir, storeName, taskModel.getTaskName(),
+          taskModel.getTaskMode());
+      StorageEngine engine = ContainerStorageManager.createStore(storeName, storeDirectory, taskModel, jobContext, containerContext,
+          storageEngineFactories, serdes, metricsRegistry, messageCollector,
+          StorageEngineFactory.StoreMode.BulkLoad, this.storeChangelogs, this.config);
+      storageEngines.put(storeName, engine);
+    });
+    return storageEngines;
+  }
+
   /**
    * Get offset metadata for each changelog SSP for this task. A task may have multiple changelog streams
    * (e.g., for different stores), but will have the same partition for all of them.
@@ -194,7 +247,8 @@ public class TransactionalStateTaskRestoreManager implements TaskRestoreManager
       TaskModel taskModel,
       Map<String, StorageEngine> storeEngines,
       Map<String, SystemStream> storeChangelogs,
-      Map<SystemStreamPartition, String> checkpointedChangelogOffsets,
+      Map<String, KafkaStateCheckpointMarker> kafkaStateCheckpointMarkers,
+      CheckpointId checkpointId,
       Map<SystemStreamPartition, SystemStreamPartitionMetadata> currentChangelogOffsets,
       SystemAdmins systemAdmins,
       StorageManagerUtil storageManagerUtil,
@@ -236,19 +290,16 @@ public class TransactionalStateTaskRestoreManager implements TaskRestoreManager
       String oldestOffset = changelogSSPMetadata.getOldestOffset();
       String newestOffset = changelogSSPMetadata.getNewestOffset();
 
-      String checkpointMessage = checkpointedChangelogOffsets.get(changelogSSP);
-      String checkpointedOffset = null;  // can be null if no message, or message has null offset
-      long timeSinceLastCheckpointInMs = Long.MAX_VALUE;
-      if (StringUtils.isNotBlank(checkpointMessage)) {
-        CheckpointedChangelogOffset checkpointedChangelogOffset = CheckpointedChangelogOffset.fromString(checkpointMessage);
-        checkpointedOffset = checkpointedChangelogOffset.getOffset();
-        timeSinceLastCheckpointInMs = System.currentTimeMillis() -
-            checkpointedChangelogOffset.getCheckpointId().getMillis();
+      String checkpointedOffset = null; // can be null if no message, or message has null offset
+      if (kafkaStateCheckpointMarkers.containsKey(storeName) &&
+          StringUtils.isNotBlank(kafkaStateCheckpointMarkers.get(storeName).getChangelogOffset())) {
+        checkpointedOffset = kafkaStateCheckpointMarkers.get(storeName).getChangelogOffset();
       }
+      long timeSinceLastCheckpointInMs = checkpointId == null ? Long.MAX_VALUE : System.currentTimeMillis() - checkpointId.getMillis();
 
       // if the clean.store.start config is set, delete current and checkpoint dirs, restore from oldest offset to checkpointed
       if (storageEngine.getStoreProperties().isPersistedToDisk() && new StorageConfig(
-        config).getCleanLoggedStoreDirsOnStart(storeName)) {
+        config).cleanLoggedStoreDirsOnStart(storeName)) {
         File currentDir = storageManagerUtil.getTaskStoreDir(loggedStoreBaseDirectory, storeName, taskName, taskMode);
         LOG.info("Marking current directory: {} for store: {} in task: {} for deletion due to clean.on.container.start config.",
             currentDir, storeName, taskName);
@@ -557,6 +608,60 @@ public class TransactionalStateTaskRestoreManager implements TaskRestoreManager
     }
   }
 
+  private Map<String, KafkaStateCheckpointMarker> getCheckpointedChangelogOffsets(Checkpoint checkpoint) {
+    Map<String, KafkaStateCheckpointMarker> checkpointedChangelogOffsets = new HashMap<>();
+    if (checkpoint == null) return checkpointedChangelogOffsets;
+
+    if (checkpoint instanceof CheckpointV2) {
+      Map<String, Map<String, String>> factoryStoreSCMs = ((CheckpointV2) checkpoint).getStateCheckpointMarkers();
+      if (factoryStoreSCMs.containsKey(KafkaStateCheckpointMarker.KAFKA_STATE_BACKEND_FACTORY_NAME)) {
+        factoryStoreSCMs.get(KafkaStateCheckpointMarker.KAFKA_STATE_BACKEND_FACTORY_NAME)
+            .forEach((storeName, scmString) -> {
+              KafkaStateCheckpointMarker kafkaSCM = KafkaStateCheckpointMarker.deserialize(scmString);
+              checkpointedChangelogOffsets.put(storeName, kafkaSCM);
+            });
+      } // skip the non-KafkaStateCheckpointMarkers
+    } else if (checkpoint instanceof CheckpointV1) {
+      // If the checkpoint v1 is used, we need to fetch the changelog SSPs in the inputOffsets in order to get the
+      // store offset.
+      Map<SystemStreamPartition, String> checkpointedOffsets = checkpoint.getOffsets();
+      storeChangelogs.forEach((storeName, systemStream) -> {
+        Partition changelogPartition = taskModel.getChangelogPartition();
+        SystemStreamPartition storeChangelogSSP = new SystemStreamPartition(systemStream, changelogPartition);
+        String checkpointedOffset = checkpointedOffsets.get(storeChangelogSSP);
+        if (StringUtils.isNotBlank(checkpointedOffset)) {
+          KafkaChangelogSSPOffset kafkaChangelogSSPOffset = KafkaChangelogSSPOffset.fromString(checkpointedOffset);
+          KafkaStateCheckpointMarker marker = new KafkaStateCheckpointMarker(
+              storeChangelogSSP, kafkaChangelogSSPOffset.getChangelogOffset());
+          checkpointedChangelogOffsets.put(storeName, marker);
+        }
+      });
+    } else {
+      throw new SamzaException("Unsupported checkpoint version: " + checkpoint.getVersion());
+    }
+
+    return checkpointedChangelogOffsets;
+  }
+
+  private CheckpointId getCheckpointId(Checkpoint checkpoint) {
+    if (checkpoint == null) return null;
+    if (checkpoint instanceof CheckpointV1) {
+      for (Map.Entry<String, SystemStream> storeNameSystemStream : storeChangelogs.entrySet()) {
+        SystemStreamPartition storeChangelogSSP = new SystemStreamPartition(storeNameSystemStream.getValue(), taskModel.getChangelogPartition());
+        String checkpointMessage = checkpoint.getOffsets().get(storeChangelogSSP);
+        if (StringUtils.isNotBlank(checkpointMessage)) {
+          KafkaChangelogSSPOffset kafkaStateChanglogOffset = KafkaChangelogSSPOffset.fromString(checkpointMessage);
+          return kafkaStateChanglogOffset.getCheckpointId();
+        }
+      }
+    } else if (checkpoint instanceof CheckpointV2) {
+      return ((CheckpointV2) checkpoint).getCheckpointId();
+    } else {
+      throw new SamzaException("Unsupported checkpoint version: " + checkpoint.getVersion());
+    }
+    return null;
+  }
+
   @VisibleForTesting
   static class StoreActions {
     final Map<String, File> storeDirsToRetain;
