@@ -27,17 +27,19 @@ import java.util
 import java.util.concurrent._
 import java.util.function.Consumer
 import java.util.{Base64, Optional}
-import com.google.common.annotations.VisibleForTesting
 import com.google.common.util.concurrent.ThreadFactoryBuilder
 import org.apache.samza.SamzaException
+import org.apache.samza.application.ApplicationUtil
 import org.apache.samza.checkpoint.{CheckpointListener, OffsetManager, OffsetManagerMetrics}
 import org.apache.samza.clustermanager.StandbyTaskUtil
 import org.apache.samza.config.{StreamConfig, _}
 import org.apache.samza.container.disk.DiskSpaceMonitor.Listener
 import org.apache.samza.container.disk.{DiskQuotaPolicyFactory, DiskSpaceMonitor, NoThrottlingDiskQuotaPolicyFactory, PollingScanDiskSpaceMonitor}
-import org.apache.samza.container.host.{StatisticsMonitorImpl, SystemMemoryStatistics, SystemStatisticsMonitor}
+import org.apache.samza.container.host.{StatisticsMonitorImpl, SystemMemoryStatistics, SystemStatistics, SystemStatisticsMonitor}
 import org.apache.samza.context._
 import org.apache.samza.diagnostics.DiagnosticsManager
+import org.apache.samza.drain.DrainMonitor.DrainCallback
+import org.apache.samza.drain.DrainMonitor
 import org.apache.samza.job.model.{ContainerModel, JobModel, TaskMode}
 import org.apache.samza.metrics.{JmxServer, JvmMetrics, MetricsRegistryMap, MetricsReporter}
 import org.apache.samza.serializers._
@@ -133,7 +135,8 @@ object SamzaContainer extends Logging {
     externalContextOption: Option[ExternalContext],
     localityManager: LocalityManager = null,
     startpointManager: StartpointManager = null,
-    diagnosticsManager: Option[DiagnosticsManager] = Option.empty) = {
+    diagnosticsManager: Option[DiagnosticsManager] = Option.empty,
+    drainMonitor: DrainMonitor = null) = {
     val config = if (StandbyTaskUtil.isStandbyContainer(containerId)) {
       // standby containers will need to continually poll checkpoint messages
       val newConfig = new util.HashMap[String, String](jobContext.getConfig)
@@ -423,6 +426,8 @@ object SamzaContainer extends Logging {
 
     val pollIntervalMs = taskConfig.getPollIntervalMs
 
+    val appConfig = new ApplicationConfig(config)
+
     val consumerMultiplexer = new SystemConsumers(
       chooser = chooser,
       consumers = consumers,
@@ -431,7 +436,9 @@ object SamzaContainer extends Logging {
       metrics = systemConsumersMetrics,
       dropDeserializationError = dropDeserializationError,
       pollIntervalMs = pollIntervalMs,
-      clock = () => clock.nanoTime())
+      clock = () => clock.nanoTime(),
+      elasticityFactor = jobConfig.getElasticityFactor,
+      runId = appConfig.getRunId)
 
     val producerMultiplexer = new SystemProducers(
       producers = producers,
@@ -456,8 +463,9 @@ object SamzaContainer extends Logging {
     samzaContainerMetrics.containerThreadPoolSize.set(threadPoolSize)
 
     val taskThreadPool = if (threadPoolSize > 0) {
-      Executors.newFixedThreadPool(threadPoolSize,
-        new ThreadFactoryBuilder().setNameFormat("Samza Container Thread-%d").build())
+      val taskExecutorFactoryClassName = jobConfig.getTaskExecutorFactory
+      val taskExecutorFactory = ReflectionUtil.getObj(taskExecutorFactoryClassName, classOf[TaskExecutorFactory])
+      taskExecutorFactory.getTaskExecutor(config)
     } else {
       null
     }
@@ -538,12 +546,10 @@ object SamzaContainer extends Logging {
       loggedStorageBaseDir,
       nonLoggedStorageBaseDir,
       serdeManager,
-      new SystemClock)
+      SystemClock.instance())
 
     storeWatchPaths.addAll(containerStorageManager.getStoreDirectoryPaths)
 
-
-
     // Create taskInstances
     val taskInstances: Map[TaskName, TaskInstance] = taskModels
       .filter(taskModel => taskModel.getTaskMode.eq(TaskMode.Active)).map(taskModel => {
@@ -566,6 +572,7 @@ object SamzaContainer extends Logging {
       info ("Got task side input SSPs: %s" format taskSideInputSSPs)
 
       val taskBackupManagerMap = new util.HashMap[String, TaskBackupManager]()
+      val systemAdminsMap = systemAdmins.getSystemAdmins
       stateStorageBackendBackupFactories.asJava.forEach(new Consumer[StateBackendFactory] {
         override def accept(factory: StateBackendFactory): Unit = {
           val taskMetricsRegistry =
@@ -573,7 +580,7 @@ object SamzaContainer extends Logging {
               taskInstanceMetrics.get(taskName).isDefined) taskInstanceMetrics.get(taskName).get.registry
             else new MetricsRegistryMap
           val taskBackupManager = factory.getBackupManager(jobContext, containerModel,
-            taskModel, commitThreadPool, taskMetricsRegistry, config, new SystemClock,
+            taskModel, systemAdminsMap, commitThreadPool, taskMetricsRegistry, config, SystemClock.instance,
             loggedStorageBaseDir, nonLoggedStorageBaseDir)
           taskBackupManagerMap.put(factory.getClass.getName, taskBackupManager)
         }
@@ -619,7 +626,9 @@ object SamzaContainer extends Logging {
 
     val maxThrottlingDelayMs = config.getLong("container.disk.quota.delay.max.ms", TimeUnit.SECONDS.toMillis(1))
 
-    val runLoop = RunLoopFactory.createRunLoop(
+    val isHighLevelApiJob = ApplicationUtil.isHighLevelApiJob(config)
+
+    val runLoop: Runnable = RunLoopFactory.createRunLoop(
       taskInstances,
       consumerMultiplexer,
       taskThreadPool,
@@ -627,22 +636,13 @@ object SamzaContainer extends Logging {
       samzaContainerMetrics,
       taskConfig,
       clock,
-      jobConfig.getElasticityFactor)
+      jobConfig.getElasticityFactor,
+      appConfig.getRunId,
+      isHighLevelApiJob)
 
-    val containerMemoryMb : Int = new ClusterManagerConfig(config).getContainerMemoryMb
-
-    val memoryStatisticsMonitor : SystemStatisticsMonitor = new StatisticsMonitorImpl()
-    memoryStatisticsMonitor.registerListener(new SystemStatisticsMonitor.Listener {
-      override def onUpdate(sample: SystemMemoryStatistics): Unit = {
-        val physicalMemoryBytes : Long = sample.getPhysicalMemoryBytes
-        val physicalMemoryMb : Float = physicalMemoryBytes / (1024.0F * 1024.0F)
-        val memoryUtilization : Float = physicalMemoryMb.toFloat / containerMemoryMb
-        logger.debug("Container physical memory utilization (mb): " + physicalMemoryMb)
-        logger.debug("Container physical memory utilization: " + memoryUtilization)
-        samzaContainerMetrics.physicalMemoryMb.set(physicalMemoryMb)
-        samzaContainerMetrics.physicalMemoryUtilization.set(memoryUtilization);
-      }
-    })
+    val systemStatisticsMonitor : SystemStatisticsMonitor = new StatisticsMonitorImpl()
+    systemStatisticsMonitor.registerListener(
+      new SamzaContainerMonitorListener(config, samzaContainerMetrics, taskThreadPool))
 
     val diskQuotaBytes = config.getLong("container.disk.quota.bytes", Long.MaxValue)
     samzaContainerMetrics.diskQuotaBytes.set(diskQuotaBytes)
@@ -669,6 +669,7 @@ object SamzaContainer extends Logging {
     } else {
       info(s"Disk quotas disabled because polling interval is not set ($DISK_POLL_INTERVAL_KEY)")
     }
+
     info("Samza container setup complete.")
 
     new SamzaContainer(
@@ -686,7 +687,7 @@ object SamzaContainer extends Logging {
       reporters = reporters,
       jvm = jvm,
       diskSpaceMonitor = diskSpaceMonitor,
-      hostStatisticsMonitor = memoryStatisticsMonitor,
+      hostStatisticsMonitor = systemStatisticsMonitor,
       taskThreadPool = taskThreadPool,
       commitThreadPool = commitThreadPool,
       timerExecutor = timerExecutor,
@@ -694,6 +695,7 @@ object SamzaContainer extends Logging {
       applicationContainerContextOption = applicationContainerContextOption,
       externalContextOption = externalContextOption,
       containerStorageManager = containerStorageManager,
+      drainMonitor = drainMonitor,
       diagnosticsManager = diagnosticsManager)
   }
 }
@@ -721,6 +723,7 @@ class SamzaContainer(
   applicationContainerContextOption: Option[ApplicationContainerContext],
   externalContextOption: Option[ExternalContext],
   containerStorageManager: ContainerStorageManager,
+  drainMonitor: DrainMonitor = null,
   diagnosticsManager: Option[DiagnosticsManager] = Option.empty) extends Runnable with Logging {
 
   private val jobConfig = new JobConfig(config)
@@ -740,6 +743,11 @@ class SamzaContainer(
 
   def getStatus(): SamzaContainerStatus = status
 
+  def drain() {
+    // set the SystemConsumers multiplexer and RunLoop in drain mode
+    consumerMultiplexer.drain
+  }
+
   def getTaskInstances() = taskInstances
 
   def setContainerListener(listener: SamzaContainerListener): Unit = {
@@ -766,6 +774,7 @@ class SamzaContainer(
       startMetrics
       startDiagnostics
       startAdmins
+      startDrainMonitor
       startOffsetManager
       storeContainerLocality
       // TODO HIGH pmaheshw SAMZA-2338: since store restore needs to trim changelog messages,
@@ -829,6 +838,7 @@ class SamzaContainer(
 
       shutdownConsumers
       shutdownTask
+      shutdownDrainMonitor
       shutdownTableManager
       shutdownStores
       shutdownDiskSpaceMonitor
@@ -1024,6 +1034,16 @@ class SamzaContainer(
     }
   }
 
+  def startDrainMonitor: Unit = {
+    if (drainMonitor != null) {
+      drainMonitor.registerDrainCallback(new DrainCallback {
+        override def onDrain(): Unit = drain()
+      })
+      info("Starting DrainMonitor.")
+      drainMonitor.start()
+    }
+  }
+
   def shutdownConsumers {
     info("Shutting down consumer multiplexer.")
 
@@ -1141,4 +1161,11 @@ class SamzaContainer(
       hostStatisticsMonitor.stop()
     }
   }
+
+  def shutdownDrainMonitor: Unit = {
+    if (drainMonitor != null) {
+      info("Shutting down DrainMonitor.")
+      drainMonitor.stop();
+    }
+  }
 }
