@@ -72,6 +72,7 @@ class TaskInstance(
 
   val taskName: TaskName = taskModel.getTaskName
   val isInitableTask = task.isInstanceOf[InitableTask]
+  val isDrainTask = task.isInstanceOf[DrainListenerTask]
   val isEndOfStreamListenerTask = task.isInstanceOf[EndOfStreamListenerTask]
   val isClosableTask = task.isInstanceOf[ClosableTask]
 
@@ -111,6 +112,12 @@ class TaskInstance(
   val streamConfig: StreamConfig = new StreamConfig(config)
   override val intermediateStreams: java.util.Set[String] = JavaConverters.setAsJavaSetConverter(streamConfig.getStreamIds.filter(streamConfig.getIsIntermediateStream)).asJava
 
+  val intermediateSSPs: Set[SystemStreamPartition] = systemStreamPartitions.filter(ssp => {
+    val systemStream = ssp.getSystemStream
+    val streamId = streamConfig.systemStreamToStreamId(systemStream)
+    intermediateStreams.contains(streamId)
+  }).toSet
+
   val streamsToDeleteCommittedMessages: Set[String] = streamConfig.getStreamIds.filter(streamConfig.getDeleteCommittedMessages).map(streamConfig.getPhysicalName).toSet
 
   val checkpointWriteVersions = new TaskConfig(config).getCheckpointWriteVersions
@@ -158,7 +165,7 @@ class TaskInstance(
         // WARNING: cleanUp is NOT optional with blob stores since this is where we reset the TTL for
         // tracked blobs. if this TTL reset is skipped, some of the blobs retained by future commits may
         // be deleted in the background by the blob store, leading to data loss.
-        debug("Cleaning up stale state from previous run for taskName: %s" format taskName)
+        info("Cleaning up stale state from previous run for taskName: %s" format taskName)
         commitManager.cleanUp(checkpointV2.getCheckpointId, checkpointV2.getStateCheckpointMarkers)
       }
     }
@@ -189,20 +196,27 @@ class TaskInstance(
   }
 
   /**
-    * Computes the starting offset for the partitions assigned to the task and registers them with the underlying {@see SystemConsumers}.
-    *
-    * Starting offset for a partition of the task is computed in the following manner:
-    *
-    * 1. If a startpoint exists for a task, system stream partition and it resolves to a offset, then the resolved offset is used as the starting offset.
-    * 2. Else, the checkpointed offset for the system stream partition is used as the starting offset.
+   * This method registers the following with the underlying {@see SystemConsumers}:
+   * a) starting offsets for all SSPs assigned to the task
+   * b) intermediate SSPs assigned to the task
+   *
+   * Starting offset for a partition of the task is computed in the following manner:
+   *
+   * 1. If a startpoint exists for a task, system stream partition and it resolves to a offset, then the resolved offset is used as the starting offset.
+   * 2. Else, the checkpointed offset for the system stream partition is used as the starting offset.
     */
   def registerConsumers() {
     debug("Registering consumers for taskName: %s" format taskName)
+
     systemStreamPartitions.foreach(systemStreamPartition => {
       val startingOffset: String = getStartingOffset(systemStreamPartition)
       consumerMultiplexer.register(systemStreamPartition, startingOffset)
       metrics.addOffsetGauge(systemStreamPartition, () => offsetManager.getLastProcessedOffset(taskName, systemStreamPartition).orNull)
     })
+
+    intermediateSSPs.foreach(ssp => {
+      consumerMultiplexer.registerIntermediateSSP(ssp)
+    })
   }
 
   def process(envelope: IncomingMessageEnvelope, coordinator: ReadableCoordinator,
@@ -241,6 +255,16 @@ class TaskInstance(
     }
   }
 
+  def drain(coordinator: ReadableCoordinator): Unit = {
+    task match {
+      case _: DrainListenerTask =>
+        exceptionHandler.maybeHandle {
+          task.asInstanceOf[DrainListenerTask].onDrain(collector, coordinator)
+        }
+      case _ =>
+    }
+  }
+
   def window(coordinator: ReadableCoordinator) {
     if (isWindowableTask) {
       trace("Windowing for taskName: %s" format taskName)
@@ -430,7 +454,7 @@ class TaskInstance(
     new Function[util.Map[String, util.Map[String, String]], CompletableFuture[Void]] {
       override def apply(uploadSCMs: util.Map[String, util.Map[String, String]]): CompletableFuture[Void] = {
         // Perform cleanup on unused checkpoints
-        debug("Cleaning up old checkpoint state for taskName: %s checkpointId: %s" format(taskName, checkpointId))
+        info("Cleaning up old checkpoint state for taskName: %s checkpointId: %s" format(taskName, checkpointId))
         val cleanUpStartTime = System.nanoTime()
         try {
           commitManager.cleanUp(checkpointId, uploadSCMs)
@@ -544,7 +568,8 @@ class TaskInstance(
     // within the keyBucket of the SSP assigned to the task.
     val incomingMessageSsp = envelope.getSystemStreamPartition(elasticityFactor)
 
-    if (IncomingMessageEnvelope.END_OF_STREAM_OFFSET.equals(envelope.getOffset)) {
+    if (IncomingMessageEnvelope.END_OF_STREAM_OFFSET.equals(envelope.getOffset)
+      || envelope.isDrain) {
       ssp2CaughtupMapping(incomingMessageSsp) = true
     } else {
       systemAdmins match {
