@@ -29,6 +29,7 @@ import org.apache.samza.context.{ExternalContext, JobContextImpl}
 import org.apache.samza.coordinator.metadatastore.{CoordinatorStreamStore, NamespaceAwareCoordinatorStreamStore}
 import org.apache.samza.coordinator.stream.messages.SetChangelogMapping
 import org.apache.samza.coordinator.{JobModelManager, MetadataResourceUtil}
+import org.apache.samza.drain.DrainMonitor
 import org.apache.samza.execution.RemoteJobPlanner
 import org.apache.samza.job.model.JobModelUtil
 import org.apache.samza.job.{StreamJob, StreamJobFactory}
@@ -50,23 +51,6 @@ class ThreadJobFactory extends StreamJobFactory with Logging {
   def getJob(submissionConfig: Config): StreamJob = {
     info("Creating a ThreadJob, which is only meant for debugging.")
     var config = submissionConfig
-    if (new JobConfig(submissionConfig).getConfigLoaderFactory.isPresent) {
-      val originalConfig = ConfigUtil.loadConfig(submissionConfig)
-
-      // Execute planning
-      val planner = new RemoteJobPlanner(ApplicationDescriptorUtil.getAppDescriptor(ApplicationUtil.fromConfig(originalConfig), originalConfig))
-      val jobConfigs = planner.prepareJobs
-
-      if (jobConfigs.size != 1) {
-        throw new SamzaException("Only single stage job is supported.")
-      }
-
-      // This is the full job config
-      config = jobConfigs.get(0)
-      // This needs to be consistent with RemoteApplicationRunner#run where JobRunner#submit to be called instead of JobRunner#run
-      CoordinatorStreamUtil.writeConfigToCoordinatorStream(config)
-      DiagnosticsUtil.createDiagnosticsStream(config)
-    }
 
     val metricsRegistry = new MetricsRegistryMap()
     val coordinatorStreamStore: CoordinatorStreamStore = new CoordinatorStreamStore(config, metricsRegistry)
@@ -91,7 +75,9 @@ class ThreadJobFactory extends StreamJobFactory with Logging {
     val metadataResourceUtil = new MetadataResourceUtil(jobModel, metricsRegistry, config)
     metadataResourceUtil.createResources()
 
-    if (new JobConfig(config).getStartpointEnabled()) {
+    val jobConfig = new JobConfig(config)
+
+    if (jobConfig.getStartpointEnabled()) {
       // fan out the startpoints
       val startpointManager = new StartpointManager(coordinatorStreamStore)
       startpointManager.start()
@@ -101,10 +87,14 @@ class ThreadJobFactory extends StreamJobFactory with Logging {
         startpointManager.stop()
       }
     }
+    var drainMonitor: DrainMonitor = null
+    if (jobConfig.getDrainMonitorEnabled()) {
+      drainMonitor = new DrainMonitor(coordinatorStreamStore, config, jobConfig.getDrainMonitorPollIntervalMillis)
+    }
 
     val containerId = "0"
     var jmxServer: JmxServer = null
-    if (new JobConfig(config).getJMXEnabled) {
+    if (jobConfig.getJMXEnabled) {
       jmxServer = new JmxServer()
     }
 
@@ -153,8 +143,8 @@ class ThreadJobFactory extends StreamJobFactory with Logging {
         JobContextImpl.fromConfigWithDefaults(config, jobModel),
         Option(appDesc.getApplicationContainerContextFactory.orElse(null)),
         Option(appDesc.getApplicationTaskContextFactory.orElse(null)),
-        buildExternalContext(config)
-      )
+        buildExternalContext(config),
+        drainMonitor = drainMonitor)
       container.setContainerListener(containerListener)
 
       val threadJob = new ThreadJob(container)
@@ -164,7 +154,6 @@ class ThreadJobFactory extends StreamJobFactory with Logging {
       if (jmxServer != null) {
         jmxServer.stop
       }
-      coordinatorStreamStore.close()
     }
   }
 
