@@ -42,9 +42,11 @@ import java.util.function.Function;
 import java.util.stream.Collectors;
 import org.apache.commons.collections4.MapUtils;
 import org.apache.samza.SamzaException;
+import org.apache.samza.application.ApplicationUtil;
 import org.apache.samza.checkpoint.Checkpoint;
 import org.apache.samza.checkpoint.CheckpointManager;
 import org.apache.samza.checkpoint.CheckpointV2;
+import org.apache.samza.config.ApplicationConfig;
 import org.apache.samza.config.Config;
 import org.apache.samza.config.JobConfig;
 import org.apache.samza.config.StorageConfig;
@@ -120,6 +122,8 @@ public class ContainerStorageManager {
 
   private static final int RESTORE_THREAD_POOL_SHUTDOWN_TIMEOUT_SECONDS = 60;
 
+  private static final int DEFAULT_SIDE_INPUT_ELASTICITY_FACTOR = 1;
+
   /** Maps containing relevant per-task objects */
   private final Map<TaskName, TaskInstanceMetrics> taskInstanceMetrics;
   private final Map<TaskName, TaskInstanceCollector> taskInstanceCollectors;
@@ -193,15 +197,15 @@ public class ContainerStorageManager {
       Clock clock) {
     this.checkpointManager = checkpointManager;
     this.containerModel = containerModel;
-    this.taskSideInputStoreSSPs = getTaskSideInputSSPs(containerModel, sideInputSystemStreams);
-    this.sideInputStoreNames = sideInputSystemStreams.keySet();
+    this.taskSideInputStoreSSPs = getTaskSideInputSSPs(containerModel, sideInputSystemStreams, changelogSystemStreams);
+    this.sideInputStoreNames = getSideInputStores(containerModel, sideInputSystemStreams, changelogSystemStreams);
     this.sideInputTaskLatches = new HashMap<>();
     this.hasSideInputs = this.taskSideInputStoreSSPs.values().stream()
         .flatMap(m -> m.values().stream())
         .flatMap(Collection::stream)
         .findAny()
         .isPresent();
-    this.changelogSystemStreams = getChangelogSystemStreams(containerModel, changelogSystemStreams); // handling standby tasks
+    this.changelogSystemStreams = getActiveTaskChangelogSystemStreams(containerModel, changelogSystemStreams);
 
     LOG.info("Starting with changelogSystemStreams = {} taskSideInputStoreSSPs = {}", this.changelogSystemStreams, this.taskSideInputStoreSSPs);
 
@@ -214,7 +218,7 @@ public class ContainerStorageManager {
 
     if (loggedStoreBaseDirectory != null && loggedStoreBaseDirectory.equals(nonLoggedStoreBaseDirectory)) {
       LOG.warn("Logged and non-logged store base directory are configured to same path: {}. It is recommended to configure"
-          + "them separately to ensure clean up of non-logged store data doesn't accidentally impact logged store data.",
+              + "them separately to ensure clean up of non-logged store data doesn't accidentally impact logged store data.",
           loggedStoreBaseDirectory);
     }
 
@@ -297,45 +301,29 @@ public class ContainerStorageManager {
       MessageChooser chooser = DefaultChooser.apply(inputStreamMetadata, new RoundRobinChooserFactory(), config,
           sideInputSystemConsumersMetrics.registry(), systemAdmins);
 
+      ApplicationConfig applicationConfig = new ApplicationConfig(config);
+
       sideInputSystemConsumers =
           new SystemConsumers(chooser, ScalaJavaUtil.toScalaMap(sideInputConsumers), systemAdmins, serdeManager,
               sideInputSystemConsumersMetrics, SystemConsumers.DEFAULT_NO_NEW_MESSAGES_TIMEOUT(), SystemConsumers.DEFAULT_DROP_SERIALIZATION_ERROR(),
-              TaskConfig.DEFAULT_POLL_INTERVAL_MS, ScalaJavaUtil.toScalaFunction(() -> System.nanoTime()));
+              TaskConfig.DEFAULT_POLL_INTERVAL_MS, ScalaJavaUtil.toScalaFunction(() -> System.nanoTime()),
+              JobConfig.DEFAULT_JOB_ELASTICITY_FACTOR, applicationConfig.getRunId());
     }
 
   }
 
   /**
-   * Add all sideInputs to a map of maps, indexed first by taskName, then by sideInput store name.
-   *
-   * @param containerModel the containerModel to use
-   * @param sideInputSystemStreams the map of store to sideInput system stream
-   * @return taskSideInputSSPs map
-   */
-  private Map<TaskName, Map<String, Set<SystemStreamPartition>>> getTaskSideInputSSPs(ContainerModel containerModel, Map<String, Set<SystemStream>> sideInputSystemStreams) {
-    Map<TaskName, Map<String, Set<SystemStreamPartition>>> taskSideInputSSPs = new HashMap<>();
-
-    containerModel.getTasks().forEach((taskName, taskModel) -> {
-      taskSideInputSSPs.putIfAbsent(taskName, new HashMap<>());
-      sideInputSystemStreams.keySet().forEach(storeName -> {
-        Set<SystemStreamPartition> taskSideInputs = taskModel.getSystemStreamPartitions().stream().filter(ssp -> sideInputSystemStreams.get(storeName).contains(ssp.getSystemStream())).collect(Collectors.toSet());
-        taskSideInputSSPs.get(taskName).put(storeName, taskSideInputs);
-      });
-    });
-    return taskSideInputSSPs;
-  }
-
-  /**
-   * For each standby task, we remove its changeLogSSPs from changelogSSP map and add it to the task's taskSideInputSSPs.
-   * The task's sideInputManager will consume and restore these as well.
+   * Remove changeLogSSPs that are associated with standby tasks from changelogSSP map and only return changelogSSPs
+   * associated with the active tasks.
+   * The standby changelogs will be consumed and restored as side inputs.
    *
    * @param containerModel the container's model
    * @param changelogSystemStreams the passed in set of changelogSystemStreams
    * @return A map of changeLogSSP to storeName across all tasks, assuming no two stores have the same changelogSSP
    */
-  private Map<String, SystemStream> getChangelogSystemStreams(ContainerModel containerModel,
+  @VisibleForTesting
+  Map<String, SystemStream> getActiveTaskChangelogSystemStreams(ContainerModel containerModel,
       Map<String, SystemStream> changelogSystemStreams) {
-
     if (MapUtils.invertMap(changelogSystemStreams).size() != changelogSystemStreams.size()) {
       throw new SamzaException("Two stores cannot have the same changelog system-stream");
     }
@@ -347,11 +335,9 @@ public class ContainerStorageManager {
     );
 
     getTasks(containerModel, TaskMode.Standby).forEach((taskName, taskModel) -> {
-      taskSideInputStoreSSPs.putIfAbsent(taskName, new HashMap<>());
       changelogSystemStreams.forEach((storeName, systemStream) -> {
         SystemStreamPartition ssp = new SystemStreamPartition(systemStream, taskModel.getChangelogPartition());
         changelogSSPToStore.remove(ssp);
-        taskSideInputStoreSSPs.get(taskName).put(storeName, Collections.singleton(ssp));
       });
     });
 
@@ -360,6 +346,58 @@ public class ContainerStorageManager {
         .collect(Collectors.toMap(Map.Entry::getKey, x -> x.getValue().getSystemStream()));
   }
 
+  /**
+   * Fetch the side input stores. For active containers, the stores correspond to the side inputs and for standbys, they
+   * include the durable stores.
+   * @param containerModel the container's model
+   * @param sideInputSystemStreams the map of store to side input system streams
+   * @param changelogSystemStreams the map of store to changelog system streams
+   * @return A set of side input stores
+   */
+  @VisibleForTesting
+  Set<String> getSideInputStores(ContainerModel containerModel,
+      Map<String, Set<SystemStream>> sideInputSystemStreams, Map<String, SystemStream> changelogSystemStreams) {
+    // add all the side input stores by default regardless of active vs standby
+    Set<String> sideInputStores = new HashSet<>(sideInputSystemStreams.keySet());
+
+    // In case of standby tasks, we treat the stores that have changelogs as side input stores for bootstrapping state
+    if (getTasks(containerModel, TaskMode.Standby).size() > 0) {
+      sideInputStores.addAll(changelogSystemStreams.keySet());
+    }
+    return sideInputStores;
+  }
+
+  /**
+   * Add all sideInputs to a map of maps, indexed first by taskName, then by sideInput store name.
+   *
+   * @param containerModel the containerModel to use
+   * @param sideInputSystemStreams the map of store to sideInput system stream
+   * @param changelogSystemStreams the map of store to changelog system stream
+   * @return taskSideInputSSPs map
+   */
+  @VisibleForTesting
+  Map<TaskName, Map<String, Set<SystemStreamPartition>>> getTaskSideInputSSPs(ContainerModel containerModel,
+      Map<String, Set<SystemStream>> sideInputSystemStreams, Map<String, SystemStream> changelogSystemStreams) {
+    Map<TaskName, Map<String, Set<SystemStreamPartition>>> taskSideInputSSPs = new HashMap<>();
+
+    containerModel.getTasks().forEach((taskName, taskModel) -> {
+      taskSideInputSSPs.putIfAbsent(taskName, new HashMap<>());
+      sideInputSystemStreams.keySet().forEach(storeName -> {
+        Set<SystemStreamPartition> taskSideInputs = taskModel.getSystemStreamPartitions().stream().filter(ssp -> sideInputSystemStreams.get(storeName).contains(ssp.getSystemStream())).collect(Collectors.toSet());
+        taskSideInputSSPs.get(taskName).put(storeName, taskSideInputs);
+      });
+    });
+
+    getTasks(containerModel, TaskMode.Standby).forEach((taskName, taskModel) -> {
+      taskSideInputSSPs.putIfAbsent(taskName, new HashMap<>());
+      changelogSystemStreams.forEach((storeName, systemStream) -> {
+        SystemStreamPartition ssp = new SystemStreamPartition(systemStream, taskModel.getChangelogPartition());
+        taskSideInputSSPs.get(taskName).put(storeName, Collections.singleton(ssp));
+      });
+    });
+
+    return taskSideInputSSPs;
+  }
 
   /**
    *  Creates SystemConsumer objects for store restoration, creating one consumer per system.
@@ -403,6 +441,11 @@ public class ContainerStorageManager {
 
     backendFactoryStoreNames.forEach((factoryName, storeNames) -> {
       StateBackendFactory factory = factories.get(factoryName);
+      if (factory == null) {
+        throw new SamzaException(
+            String.format("Required restore state backend factory: %s not found in configured factories %s",
+                factoryName, String.join(", ", factories.keySet())));
+      }
       KafkaChangelogRestoreParams kafkaChangelogRestoreParams = new KafkaChangelogRestoreParams(storeConsumers,
           inMemoryStores.get(taskName), systemAdmins.getSystemAdmins(), storageEngineFactories, serdes,
           taskInstanceCollectors.get(taskName));
@@ -487,7 +530,7 @@ public class ContainerStorageManager {
   // Helper method to filter active Tasks from the container model
   private static Map<TaskName, TaskModel> getTasks(ContainerModel containerModel, TaskMode taskMode) {
     return containerModel.getTasks().entrySet().stream()
-            .filter(x -> x.getValue().getTaskMode().equals(taskMode)).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
+        .filter(x -> x.getValue().getTaskMode().equals(taskMode)).collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
   }
 
   /**
@@ -890,6 +933,8 @@ public class ContainerStorageManager {
         new SamzaContainerMetrics(SIDEINPUTS_METRICS_PREFIX + this.samzaContainerMetrics.source(),
             this.samzaContainerMetrics.registry(), SIDEINPUTS_METRICS_PREFIX);
 
+    final ApplicationConfig applicationConfig = new ApplicationConfig(config);
+
     this.sideInputRunLoop = new RunLoop(sideInputTasks,
         null, // all operations are executed in the main runloop thread
         this.sideInputSystemConsumers,
@@ -897,12 +942,17 @@ public class ContainerStorageManager {
         -1, // no windowing
         taskConfig.getCommitMs(),
         taskConfig.getCallbackTimeoutMs(),
+        taskConfig.getDrainCallbackTimeoutMs(),
         // TODO consolidate these container configs SAMZA-2275
         this.config.getLong("container.disk.quota.delay.max.ms", TimeUnit.SECONDS.toMillis(1)),
         taskConfig.getMaxIdleMs(),
         sideInputContainerMetrics,
         System::nanoTime,
-        false); // commit must be synchronous to ensure integrity of state flush
+        false,
+        DEFAULT_SIDE_INPUT_ELASTICITY_FACTOR,
+        applicationConfig.getRunId(),
+        ApplicationUtil.isHighLevelApiJob(config)
+        ); // commit must be synchronous to ensure integrity of state flush
 
     try {
       sideInputsExecutor.submit(() -> {
