@@ -28,7 +28,8 @@ import java.util.HashMap
 import java.util.HashSet
 import java.util.Queue
 import java.util.Set
-
+import java.util.function.{Consumer}
+import java.util.stream.Collectors
 import scala.collection.JavaConverters._
 import org.apache.samza.serializers.SerdeManager
 import org.apache.samza.util.{Logging, TimerUtil}
@@ -36,6 +37,7 @@ import org.apache.samza.system.chooser.MessageChooser
 import org.apache.samza.SamzaException
 import org.apache.samza.config.TaskConfig
 
+
 object SystemConsumers {
   val DEFAULT_NO_NEW_MESSAGES_TIMEOUT = 10
   val DEFAULT_DROP_SERIALIZATION_ERROR = false
@@ -112,13 +114,31 @@ class SystemConsumers (
    * Clock can be used to inject a custom clock when mocking this class in
    * tests. The default implementation returns the current system clock time.
    */
-  val clock: () => Long = () => System.nanoTime()) extends Logging with TimerUtil {
+  val clock: () => Long = () => System.nanoTime(),
+
+  val elasticityFactor: Int = 1,
+
+  /**
+   * Identifier of the current deployment.
+   * */
+  val runId: String = null) extends Logging with TimerUtil {
 
   /**
    * Mapping from the {@see SystemStreamPartition} to the registered offsets.
    */
   private val sspToRegisteredOffsets = new HashMap[SystemStreamPartition, String]()
 
+  /**
+   * Set of all the SystemStreamPartitions registered with this SystemConsumers
+   * With elasticity-enabled, the SSPs have valid (i.e. >=0) keyBuckets,
+   * with elasticity disabled, keyBuckets on all SSPs are = -1.
+   */
+  private val sspKeyBucketsRegistered = new HashSet[SystemStreamPartition] ()
+
+  private val intermediateSSPs = new HashSet[SystemStreamPartition]()
+
+  private val intermediateSystems = new HashSet[String]()
+
   /**
    * A buffer of incoming messages grouped by SystemStreamPartition. These
    * messages are handed out to the MessageChooser as it needs them.
@@ -144,6 +164,9 @@ class SystemConsumers (
     */
   private var started = false
 
+  @volatile
+  private var isDraining = false
+
   /**
    * Default timeout to noNewMessagesTimeout. Every time SystemConsumers
    * receives incoming messages, it sets timeout to 0. Every time
@@ -162,6 +185,7 @@ class SystemConsumers (
    */
   var totalUnprocessedMessages = 0
 
+  info("Got elasticity factor: %s" format elasticityFactor)
   debug("Got stream consumers: %s" format consumers)
   debug("Got no new message timeout: %s" format noNewMessagesTimeout)
 
@@ -195,6 +219,11 @@ class SystemConsumers (
 
     chooser.start
 
+    // SystemConsumers could be set to drain mode prior to start if a drain message was encountered on container start
+    if (isDraining) {
+      writeDrainControlMessageToSspQueue()
+    }
+
     started = true
 
     refresh
@@ -214,11 +243,21 @@ class SystemConsumers (
     }
   }
 
+  def drain(): Unit = {
+    if (!isDraining) {
+      isDraining = true;
+      info("SystemConsumers is set to drain mode.")
+      if (started) {
+        writeDrainControlMessageToSspQueue()
+      }
+    }
+  }
 
   def register(ssp: SystemStreamPartition, offset: String) {
     // If elasticity is enabled then the RunLoop gives SSP with keybucket
     // but the MessageChooser does not know about the KeyBucket
     // hence, use an SSP without KeyBucket
+    sspKeyBucketsRegistered.add(ssp)
     val systemStreamPartition = removeKeyBucket(ssp)
     debug("Registering stream: %s, %s" format (systemStreamPartition, offset))
 
@@ -246,12 +285,17 @@ class SystemConsumers (
     }
   }
 
+  def registerIntermediateSSP(ssp: SystemStreamPartition): Unit = {
+    debug("Registering intermediate stream: %s" format ssp)
+    intermediateSSPs.add(ssp)
+    intermediateSystems.add(ssp.getSystem)
+  }
 
   def isEndOfStream(systemStreamPartition: SystemStreamPartition) = {
     endOfStreamSSPs.contains(removeKeyBucket(systemStreamPartition))
   }
 
-  def choose (updateChooser: Boolean = true): IncomingMessageEnvelope = {
+  def choose(updateChooser: Boolean = true): IncomingMessageEnvelope = {
     val envelopeFromChooser = chooser.choose
 
     updateTimer(metrics.deserializationNs) {
@@ -277,9 +321,18 @@ class SystemConsumers (
 
         // Ok to give the chooser a new message from this stream.
         timeout = 0
-        metrics.choseObject.inc
-        metrics.systemStreamMessagesChosen(envelopeFromChooser.getSystemStreamPartition).inc
-
+        if (elasticityFactor == 1) {
+          metrics.choseObject.inc
+          metrics.systemStreamMessagesChosen(envelopeFromChooser.getSystemStreamPartition).inc
+        } else {
+          // increment metrics only if the envelope belongs to one of the SSP key buckets registered with this SystemConsumers
+          if (sspKeyBucketsRegistered.contains(envelopeFromChooser.getSystemStreamPartition(elasticityFactor))) {
+            metrics.choseObject.inc
+            metrics.systemStreamMessagesChosen(envelopeFromChooser.getSystemStreamPartition).inc
+          } else {
+            metrics.choseNull.inc
+          }
+        }
         if (updateChooser) {
           trace("Update chooser for " + systemStreamPartition.getPartition)
           tryUpdate(systemStreamPartition)
@@ -367,12 +420,54 @@ class SystemConsumers (
   }
 
   private def refresh {
-    trace("Refreshing chooser with new messages.")
-
     // Update last poll time so we don't poll too frequently.
     lastPollNs = clock()
-    // Poll every system for new messages.
-    consumers.keys.map(poll(_))
+
+    if (isDraining) {
+      trace("Refreshing chooser with new messages from intermediate systems.")
+
+      // scala 2.11 doesn't allow using syntactical sugar: intermediateSystems.foreach(poll(_)) over java collections
+      intermediateSystems.forEach(new Consumer[String] {
+        override def accept(system: String): Unit = poll(system)
+      })
+    } else {
+      trace("Refreshing chooser with new messages.")
+      consumers.keys.foreach(poll(_))
+    }
+  }
+
+  private def writeDrainControlMessageToSspQueue() {
+    val sspsToDrain = new HashSet(sspKeyBucketsRegistered)
+
+    // only write Drain ControlMessages to source SSPs
+    // sspsToDrain = allSSPs - intermediateSSPs - eosSSPs
+    sspsToDrain.removeAll(intermediateSSPs)
+    sspsToDrain.removeAll(endOfStreamSSPs)
+
+    sspsToDrain.forEach(new Consumer[SystemStreamPartition] {
+      override def accept(ssp: SystemStreamPartition): Unit = {
+        val envelopes: Queue[IncomingMessageEnvelope] =
+          if (unprocessedMessagesBySSP.containsKey(ssp)) {
+            unprocessedMessagesBySSP.get(ssp)
+          } else {
+            new util.ArrayDeque[IncomingMessageEnvelope]()
+          }
+
+        // Add watermark ControlMessage only if there are intermediate SSPs as low-level API task doesn't process
+        // WatermarkMessages
+        if (!intermediateSSPs.isEmpty) {
+          envelopes.add(IncomingMessageEnvelope.buildWatermarkEnvelope(ssp, Long.MaxValue))
+          totalUnprocessedMessages += 1
+        }
+        // Add Drain ControlMessage
+        envelopes.add(IncomingMessageEnvelope.buildDrainMessage(ssp, runId))
+        totalUnprocessedMessages += 1
+        unprocessedMessagesBySSP.put(ssp, envelopes)
+
+        // update the chooser with the messages
+        tryUpdate(ssp)
+      }
+    })
   }
 
   /**
