@@ -28,6 +28,7 @@ import java.util.Set;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicInteger;
 import java.util.stream.Collectors;
+import com.google.common.collect.ImmutableSet;
 import org.apache.samza.clustermanager.container.placement.ContainerPlacementMetadataStore;
 import org.apache.samza.config.Config;
 import org.apache.samza.config.MapConfig;
@@ -410,7 +411,7 @@ public class TestContainerAllocatorWithHostAffinity {
     // State check when host affinity is enabled
     assertTrue(state.matchedResourceRequests.get() == 2);
     assertTrue(state.preferredHostRequests.get() == 2);
-    containerAllocator.stop();
+    spyAllocator.stop();
   }
 
   @Test
@@ -432,28 +433,38 @@ public class TestContainerAllocatorWithHostAffinity {
     // Start the container allocator thread periodic assignment
     spyAllocatorThread.start();
 
-    // Let the request expire, expiration timeout is 3 ms
-    Thread.sleep(100);
+    // Let the preferred host requests and the follow-up ANY_HOST request expire, expiration timeout is 500 ms
+    Thread.sleep(1500);
 
     // Verify that all the request that were created as preferred host requests expired
     assertTrue(state.preferredHostRequests.get() == 2);
     assertTrue(state.expiredPreferredHostRequests.get() == 2);
-    verify(spyContainerManager, times(1)).handleExpiredRequest(eq("0"), eq("hostname-0"),
+    // expirations for initial preferred host requests
+    verify(spyContainerManager).handleExpiredRequest(eq("0"), eq("hostname-0"),
         any(SamzaResourceRequest.class), any(ContainerAllocator.class), any(ResourceRequestState.class));
-    verify(spyContainerManager, times(1)).handleExpiredRequest(eq("1"), eq("hostname-1"),
+    verify(spyContainerManager).handleExpiredRequest(eq("1"), eq("hostname-1"),
+        any(SamzaResourceRequest.class), any(ContainerAllocator.class), any(ResourceRequestState.class));
+    // expirations for follow-up ANY_HOST requests
+    // allocator keeps running in a loop so it might expire more than once during the wait time
+    verify(spyContainerManager, atLeast(1)).handleExpiredRequest(eq("0"), eq(ResourceRequestState.ANY_HOST),
+        any(SamzaResourceRequest.class), any(ContainerAllocator.class), any(ResourceRequestState.class));
+    verify(spyContainerManager, atLeast(1)).handleExpiredRequest(eq("1"), eq(ResourceRequestState.ANY_HOST),
         any(SamzaResourceRequest.class), any(ContainerAllocator.class), any(ResourceRequestState.class));
 
     // Verify that preferred host request were cancelled and since no surplus resources were available
     // requestResource was invoked with ANY_HOST requests
     ArgumentCaptor<SamzaResourceRequest> cancelledRequestCaptor = ArgumentCaptor.forClass(SamzaResourceRequest.class);
-    // At least 2 preferred host requests were cancelled
-    verify(spyManager, atLeast(2)).cancelResourceRequest(cancelledRequestCaptor.capture());
-    assertTrue(cancelledRequestCaptor.getAllValues().stream().map(resourceRequest -> resourceRequest.getPreferredHost()).collect(
-        Collectors.toSet()).size() > 2);
-    // Check that atleast 2 ANY_HOST requests were made
+    // preferred host requests and ANY_HOST requests got cancelled
+    verify(spyManager, atLeast(4)).cancelResourceRequest(cancelledRequestCaptor.capture());
+    assertEquals(ImmutableSet.of("hostname-0", "hostname-1", ResourceRequestState.ANY_HOST),
+        cancelledRequestCaptor.getAllValues()
+            .stream()
+            .map(SamzaResourceRequest::getPreferredHost)
+            .collect(Collectors.toSet()));
     assertTrue(state.matchedResourceRequests.get() == 0);
-    assertTrue(state.anyHostRequests.get() > 2);
-    containerAllocator.stop();
+    // at least 2 for expired preferred host requests, 2 for expired follow-up ANY_HOST requests
+    assertTrue(state.anyHostRequests.get() >= 4);
+    spyAllocator.stop();
   }
 
   @Test
@@ -480,8 +491,8 @@ public class TestContainerAllocatorWithHostAffinity {
     // Start the container allocator thread periodic assignment
     spyAllocatorThread.start();
 
-    // Let the request expire, expiration timeout is 3 ms
-    Thread.sleep(100);
+    // Let the request expire, expiration timeout is 500 ms
+    Thread.sleep(1000);
 
     // Verify that all the request that were created as preferred host requests expired
     assertEquals(state.expiredPreferredHostRequests.get(), 2);
@@ -506,7 +517,7 @@ public class TestContainerAllocatorWithHostAffinity {
     assertTrue(state.matchedResourceRequests.get() == 0);
     assertTrue(state.preferredHostRequests.get() == 2);
     assertTrue(state.anyHostRequests.get() == 0);
-    containerAllocator.stop();
+    spyAllocator.stop();
   }
 
   @Test(timeout = 5000)
@@ -551,7 +562,7 @@ public class TestContainerAllocatorWithHostAffinity {
         .forEach(resource -> assertEquals(resource.getHost(), "host-0"));
     // Verify resources were released
     assertTrue(mockClusterResourceManager.containsReleasedResource(expiredAllocatedResource));
-    containerAllocator.stop();
+    spyAllocator.stop();
   }
 
   //@Test
@@ -643,7 +654,7 @@ public class TestContainerAllocatorWithHostAffinity {
         put("cluster-manager.container.count", "1");
         put("cluster-manager.container.retry.count", "1");
         put("cluster-manager.container.retry.window.ms", "1999999999");
-        put("cluster-manager.container.request.timeout.ms", "3");
+        put("cluster-manager.container.request.timeout.ms", "500");
         put("cluster-manager.allocator.sleep.ms", "1");
         put("cluster-manager.container.memory.mb", "512");
         put("yarn.package.path", "/foo");
