@@ -18,11 +18,17 @@
  */
 package org.apache.samza.runtime;
 
+import com.google.common.collect.ImmutableMap;
 import java.time.Duration;
-import org.apache.samza.application.SamzaApplication;
+import joptsimple.OptionSet;
 import org.apache.samza.application.MockStreamApplication;
+import org.apache.samza.application.SamzaApplication;
 import org.apache.samza.config.ApplicationConfig;
 import org.apache.samza.config.Config;
+import org.apache.samza.config.ConfigLoaderFactory;
+import org.apache.samza.config.JobConfig;
+import org.apache.samza.config.MapConfig;
+import org.apache.samza.context.ExternalContext;
 import org.apache.samza.job.ApplicationStatus;
 import org.junit.Test;
 
@@ -32,30 +38,26 @@ import static org.junit.Assert.assertEquals;
 public class TestApplicationRunnerMain {
 
   @Test
-  public void TestRunOperation() throws Exception {
+  public void TestRunOperation() {
     assertEquals(0, TestApplicationRunnerInvocationCounts.runCount);
     ApplicationRunnerMain.main(new String[]{
-        "--config-factory",
-        "org.apache.samza.config.factories.PropertiesConfigFactory",
-        "--config-path",
-        getClass().getResource("/test.properties").getPath(),
+        "-config", "job.config.loader.factory=org.apache.samza.config.loaders.PropertiesConfigLoaderFactory",
+        "-config", "job.config.loader.properties.path=" + getClass().getResource("/test.properties").getPath(),
         "-config", String.format("%s=%s", ApplicationConfig.APP_CLASS, MockStreamApplication.class.getName()),
-        "-config", String.format("app.runner.class=%s", TestApplicationRunnerInvocationCounts.class.getName()),
+        "-config", String.format("%s=%s", ApplicationConfig.APP_RUNNER_CLASS, TestApplicationRunnerInvocationCounts.class.getName()),
     });
 
     assertEquals(1, TestApplicationRunnerInvocationCounts.runCount);
   }
 
   @Test
-  public void TestKillOperation() throws Exception {
+  public void TestKillOperation() {
     assertEquals(0, TestApplicationRunnerInvocationCounts.killCount);
     ApplicationRunnerMain.main(new String[]{
-        "--config-factory",
-        "org.apache.samza.config.factories.PropertiesConfigFactory",
-        "--config-path",
-        getClass().getResource("/test.properties").getPath(),
+        "-config", "job.config.loader.factory=org.apache.samza.config.loaders.PropertiesConfigLoaderFactory",
+        "-config", "job.config.loader.properties.path=" + getClass().getResource("/test.properties").getPath(),
         "-config", String.format("%s=%s", ApplicationConfig.APP_CLASS, MockStreamApplication.class.getName()),
-        "-config", String.format("app.runner.class=%s", TestApplicationRunnerInvocationCounts.class.getName()),
+        "-config", String.format("%s=%s", ApplicationConfig.APP_RUNNER_CLASS, TestApplicationRunnerInvocationCounts.class.getName()),
         "--operation=kill"
     });
 
@@ -63,21 +65,39 @@ public class TestApplicationRunnerMain {
   }
 
   @Test
-  public void TestStatusOperation() throws Exception {
+  public void TestStatusOperation() {
     assertEquals(0, TestApplicationRunnerInvocationCounts.statusCount);
     ApplicationRunnerMain.main(new String[]{
-        "--config-factory",
-        "org.apache.samza.config.factories.PropertiesConfigFactory",
-        "--config-path",
-        getClass().getResource("/test.properties").getPath(),
+        "-config", "job.config.loader.factory=org.apache.samza.config.loaders.PropertiesConfigLoaderFactory",
+        "-config", "job.config.loader.properties.path=" + getClass().getResource("/test.properties").getPath(),
         "-config", String.format("%s=%s", ApplicationConfig.APP_CLASS, MockStreamApplication.class.getName()),
-        "-config", String.format("app.runner.class=%s", TestApplicationRunnerInvocationCounts.class.getName()),
+        "-config", String.format("%s=%s", ApplicationConfig.APP_RUNNER_CLASS, TestApplicationRunnerInvocationCounts.class.getName()),
         "--operation=status"
     });
 
     assertEquals(1, TestApplicationRunnerInvocationCounts.statusCount);
   }
 
+  @Test
+  public void TestLoadConfig() {
+    ApplicationRunnerMain.ApplicationRunnerCommandLine cmdLine = new ApplicationRunnerMain.ApplicationRunnerCommandLine();
+    OptionSet options = cmdLine.parser().parse(
+        "-config", "job.config.loader.factory=org.apache.samza.config.loaders.PropertiesConfigLoaderFactory",
+        "-config", "job.config.loader.properties.path=" + getClass().getResource("/test.properties").getPath(),
+        "-config", String.format("%s=%s", ApplicationConfig.APP_CLASS, MockStreamApplication.class.getName()),
+        "-config", String.format("%s=%s", ApplicationConfig.APP_RUNNER_CLASS, TestApplicationRunnerInvocationCounts.class.getName()));
+
+    Config actual = cmdLine.loadConfig(options);
+
+    Config expected = new MapConfig(ImmutableMap.of(
+        JobConfig.CONFIG_LOADER_FACTORY, "org.apache.samza.config.loaders.PropertiesConfigLoaderFactory",
+        ConfigLoaderFactory.CONFIG_LOADER_PROPERTIES_PREFIX + "path", getClass().getResource("/test.properties").getPath(),
+        ApplicationConfig.APP_CLASS, MockStreamApplication.class.getName(),
+        ApplicationConfig.APP_RUNNER_CLASS, TestApplicationRunnerInvocationCounts.class.getName()));
+
+    assertEquals(expected, actual);
+  }
+
   public static class TestApplicationRunnerInvocationCounts implements ApplicationRunner {
     protected static int runCount = 0;
     protected static int killCount = 0;
@@ -87,7 +107,7 @@ public class TestApplicationRunnerMain {
     }
 
     @Override
-    public void run() {
+    public void run(ExternalContext externalContext) {
       runCount++;
     }
 
