@@ -27,6 +27,7 @@ import java.util.Collections;
 import java.util.List;
 import java.util.Optional;
 import org.apache.samza.checkpoint.CheckpointId;
+import org.apache.samza.context.Context;
 import org.apache.samza.system.ChangelogSSPIterator;
 import org.apache.samza.system.IncomingMessageEnvelope;
 import org.apache.samza.system.SystemStreamPartition;
@@ -59,6 +60,10 @@ public class MockStorageEngine implements StorageEngine {
     }
   }
 
+  @Override
+  public void init(Context context) {
+  }
+
   @Override
   public void flush() {
   }
