@@ -102,11 +102,11 @@ public class TestBlobStoreUtil {
     BlobStoreManager blobStoreManager = mock(BlobStoreManager.class);
 
     // File, dir and recursive dir added, retained and removed in local
-    String local = "[a, c, z/1, y/1, p/m/1, q/n/1]";
-    String remote = "[a, b, z/1, x/1, p/m/1, p/m/2, r/o/1]";
-    String expectedAdded = "[c, y/1, q/n/1]";
-    String expectedRetained = "[a, z/1, p/m/1]";
-    String expectedRemoved = "[b, x/1, r/o/1, p/m/2]";
+    String local = "[a, c, z/1, y/2, p/m/3, q/n/4]";
+    String remote = "[a, b, z/1, x/2, p/m/3, p/m/5, r/o/6]";
+    String expectedAdded = "[c, y/2, q/n/4]";
+    String expectedRetained = "[a, z/1, p/m/3]";
+    String expectedRemoved = "[b, x/2, r/o/6, p/m/5]";
     SortedSet<String> expectedAddedFiles = BlobStoreTestUtil.getExpected(expectedAdded);
     SortedSet<String> expectedRetainedFiles = BlobStoreTestUtil.getExpected(expectedRetained);
     SortedSet<String> expectedPresentFiles = new TreeSet<>(expectedAddedFiles);
@@ -960,6 +960,34 @@ public class TestBlobStoreUtil {
     verify(mockBlobStoreUtil, times(storesToBackupOrRestore.size())).getSnapshotIndex(anyString(), any(Metadata.class));
   }
 
+  /**
+   * This test verifies that a retriable exception is retried more than 3 times (default retry is limited to 3 attempts)
+   */
+  @Test
+  public void testPutFileRetriedMorethanThreeTimes() throws Exception {
+    SnapshotMetadata snapshotMetadata = new SnapshotMetadata(checkpointId, jobName, jobId, taskName, storeName);
+    Path path = Files.createTempFile("samza-testPutFileChecksum-", ".tmp");
+    FileUtil fileUtil = new FileUtil();
+    fileUtil.writeToTextFile(path.toFile(), RandomStringUtils.random(1000), false);
+
+    BlobStoreManager blobStoreManager = mock(BlobStoreManager.class);
+    ArgumentCaptor<Metadata> argumentCaptor = ArgumentCaptor.forClass(Metadata.class);
+    when(blobStoreManager.put(any(InputStream.class), argumentCaptor.capture()))
+        .thenAnswer((Answer<CompletionStage<String>>) invocationOnMock -> { // first try, retriable error
+          return FutureUtil.failedFuture(new RetriableException()); // retriable error
+        }).thenAnswer((Answer<CompletionStage<String>>) invocationOnMock -> { // second try, retriable error
+          return FutureUtil.failedFuture(new RetriableException()); // retriable error
+        }).thenAnswer((Answer<CompletionStage<String>>) invocationOnMock -> { // third try, retriable error
+          return FutureUtil.failedFuture(new RetriableException()); // retriable error
+        }).thenAnswer((Answer<CompletionStage<String>>) invocation -> CompletableFuture.completedFuture("blobId"));
+
+    BlobStoreUtil blobStoreUtil = new BlobStoreUtil(blobStoreManager, EXECUTOR, null, null);
+
+    blobStoreUtil.putFile(path.toFile(), snapshotMetadata).join();
+    // Verify put operation is retried 4 times
+    verify(blobStoreManager, times(4)).put(any(InputStream.class), any(Metadata.class));
+  }
+
   private CheckpointV2 createCheckpointV2(String stateBackendFactory, Map<String, String> storeSnapshotIndexBlobIds) {
     CheckpointId checkpointId = CheckpointId.create();
     Map<String, Map<String, String>> factoryStoreSCMs = new HashMap<>();
