@@ -34,7 +34,7 @@ import org.apache.samza.metrics.Timer;
 import org.apache.samza.table.descriptors.BaseTableDescriptor;
 import org.apache.samza.table.descriptors.TableDescriptor;
 import org.apache.samza.storage.kv.Entry;
-import org.apache.samza.table.ReadWriteTable;
+import org.apache.samza.table.ReadWriteUpdateTable;
 import org.apache.samza.table.caching.guava.GuavaCacheTable;
 import org.apache.samza.table.descriptors.CachingTableDescriptor;
 import org.apache.samza.table.descriptors.GuavaCacheTableDescriptor;
@@ -109,13 +109,13 @@ public class TestCachingTable {
     assertEquals("true", CachingTableDescriptor.WRITE_AROUND, "1", tableConfig);
   }
 
-  private static Pair<ReadWriteTable<String, String>, Map<String, String>> getMockCache() {
+  private static Pair<ReadWriteUpdateTable<String, String, String>, Map<String, String>> getMockCache() {
     // To allow concurrent writes for disjoint keys by testConcurrentAccess, we must use CHM here.
     // This is okay because the atomic section in CachingTable covers both cache and table so using
     // CHM for each does not serialize such two-step operation so the atomicity is still tested.
     // Regular HashMap is not thread-safe even for disjoint keys.
     final Map<String, String> cacheStore = new ConcurrentHashMap<>();
-    final ReadWriteTable cacheTable = mock(ReadWriteTable.class);
+    final ReadWriteUpdateTable cacheTable = mock(ReadWriteUpdateTable.class);
 
     doAnswer(invocation -> {
       String key = invocation.getArgumentAt(0, String.class);
@@ -137,11 +137,11 @@ public class TestCachingTable {
     return Pair.of(cacheTable, cacheStore);
   }
 
-  private void initTables(ReadWriteTable ... tables) {
+  private void initTables(ReadWriteUpdateTable... tables) {
     initTables(false, tables);
   }
 
-  private void initTables(boolean isTimerMetricsDisabled, ReadWriteTable ... tables) {
+  private void initTables(boolean isTimerMetricsDisabled, ReadWriteUpdateTable... tables) {
     Map<String, String> config = new HashMap<>();
     if (isTimerMetricsDisabled) {
       config.put(MetricsConfig.METRICS_TIMER_ENABLED, "false");
@@ -166,9 +166,9 @@ public class TestCachingTable {
     }
 
     Context context = new MockContext();
-    final ReadWriteTable cacheTable = getMockCache().getLeft();
+    final ReadWriteUpdateTable cacheTable = getMockCache().getLeft();
 
-    final ReadWriteTable realTable = mock(ReadWriteTable.class);
+    final ReadWriteUpdateTable realTable = mock(ReadWriteUpdateTable.class);
 
     doAnswer(invocation -> {
       String key = invocation.getArgumentAt(0, String.class);
@@ -188,7 +188,7 @@ public class TestCachingTable {
 
       Assert.fail();
       return null;
-    }).when(context.getTaskContext()).getTable(anyString());
+    }).when(context.getTaskContext()).getUpdatableTable(anyString());
 
     when(context.getContainerContext().getContainerMetricsRegistry()).thenReturn(new NoOpMetricsRegistry());
 
@@ -241,10 +241,10 @@ public class TestCachingTable {
 
   @Test
   public void testNonexistentKeyInTable() {
-    ReadWriteTable<String, String> table = mock(ReadWriteTable.class);
+    ReadWriteUpdateTable<String, String, String> table = mock(ReadWriteUpdateTable.class);
     doReturn(CompletableFuture.completedFuture(null)).when(table).getAsync(any());
-    ReadWriteTable<String, String> cache = getMockCache().getLeft();
-    CachingTable<String, String> cachingTable = new CachingTable<>("myTable", table, cache, false);
+    ReadWriteUpdateTable<String, String, String> cache = getMockCache().getLeft();
+    CachingTable<String, String, String> cachingTable = new CachingTable<>("myTable", table, cache, false);
     initTables(cachingTable);
     Assert.assertNull(cachingTable.get("abc"));
     verify(cache, times(1)).get(any());
@@ -254,12 +254,12 @@ public class TestCachingTable {
 
   @Test
   public void testKeyEviction() {
-    ReadWriteTable<String, String> table = mock(ReadWriteTable.class);
+    ReadWriteUpdateTable<String, String, Void> table = mock(ReadWriteUpdateTable.class);
     doReturn(CompletableFuture.completedFuture("3")).when(table).getAsync(any());
-    ReadWriteTable<String, String> cache = mock(ReadWriteTable.class);
+    ReadWriteUpdateTable<String, String, Void> cache = mock(ReadWriteUpdateTable.class);
 
     // no handler added to mock cache so get/put are noop, this can simulate eviction
-    CachingTable<String, String> cachingTable = new CachingTable<>("myTable", table, cache, false);
+    CachingTable<String, String, Void> cachingTable = new CachingTable<>("myTable", table, cache, false);
     initTables(cachingTable);
     cachingTable.get("abc");
     verify(table, times(1)).getAsync(any());
@@ -276,33 +276,34 @@ public class TestCachingTable {
   public void testGuavaCacheAndRemoteTable() throws Exception {
     String tableId = "testGuavaCacheAndRemoteTable";
     Cache<String, String> guavaCache = CacheBuilder.newBuilder().initialCapacity(100).build();
-    final ReadWriteTable<String, String> guavaTable = new GuavaCacheTable<>(tableId + "-cache", guavaCache);
+    final ReadWriteUpdateTable<String, String, String> guavaTable = new GuavaCacheTable<>(tableId + "-cache", guavaCache);
 
     // It is okay to share rateLimitHelper and async helper for read/write in test
-    TableRateLimiter<String, String> rateLimitHelper = mock(TableRateLimiter.class);
+    TableRateLimiter<String, String> readRateLimitHelper = mock(TableRateLimiter.class);
+    TableRateLimiter<String, String> writeRateLimitHelper = mock(TableRateLimiter.class);
+    TableRateLimiter<String, String> updateRateLimitHelper = mock(TableRateLimiter.class);
     TableReadFunction<String, String> readFn = mock(TableReadFunction.class);
-    TableWriteFunction<String, String> writeFn = mock(TableWriteFunction.class);
-    final RemoteTable<String, String> remoteTable = new RemoteTable<>(
-        tableId + "-remote", readFn, writeFn,
-        rateLimitHelper, rateLimitHelper, Executors.newSingleThreadExecutor(),
-        null, null, null,
-        null, null,
-        Executors.newSingleThreadExecutor());
+    TableWriteFunction<String, String, String> writeFn = mock(TableWriteFunction.class);
 
-    final CachingTable<String, String> cachingTable = new CachingTable<>(
+    final RemoteTable<String, String, String> remoteTable = new RemoteTable<>(tableId + "-remote", readFn,
+        writeFn, readRateLimitHelper, writeRateLimitHelper, updateRateLimitHelper,
+        Executors.newSingleThreadExecutor(), null, null, null, null,
+        null,  Executors.newSingleThreadExecutor());
+
+    final CachingTable<String, String, String> cachingTable = new CachingTable<>(
         tableId, remoteTable, guavaTable, false);
 
     initTables(cachingTable, guavaTable, remoteTable);
 
     // 4 per readable table (12)
-    // 6 per read/write table (18)
-    verify(metricsRegistry, times(30)).newCounter(any(), anyString());
+    // 8 per read/write table (24)
+    verify(metricsRegistry, times(36)).newCounter(any(), anyString());
 
     // 3 per readable table (9)
-    // 6 per read/write table (18)
+    // 8 per read/write table (24)
     // 1 per remote readable table (1)
-    // 1 per remote read/write table (1)
-    verify(metricsRegistry, times(29)).newTimer(any(), anyString());
+    // 2 per remote read/write table (2)
+    verify(metricsRegistry, times(36)).newTimer(any(), anyString());
 
     // 1 per guava table (1)
     // 3 per caching table (2)
@@ -393,25 +394,24 @@ public class TestCachingTable {
     String tableId = "testTimerDisabled";
 
     Cache<String, String> guavaCache = CacheBuilder.newBuilder().initialCapacity(100).build();
-    final ReadWriteTable<String, String> guavaTable = new GuavaCacheTable<>(tableId, guavaCache);
-
-    TableRateLimiter<String, String> rateLimitHelper = mock(TableRateLimiter.class);
+    final ReadWriteUpdateTable<String, String, String> guavaTable = new GuavaCacheTable<>(tableId, guavaCache);
 
+    TableRateLimiter<String, String> readRateLimitHelper = mock(TableRateLimiter.class);
+    TableRateLimiter<String, String> writeRateLimitHelper = mock(TableRateLimiter.class);
+    TableRateLimiter<String, String> updateRateLimitHelper = mock(TableRateLimiter.class);
     TableReadFunction<String, String> readFn = mock(TableReadFunction.class);
     doReturn(CompletableFuture.completedFuture("")).when(readFn).getAsync(any());
 
-    TableWriteFunction<String, String> writeFn = mock(TableWriteFunction.class);
+    TableWriteFunction<String, String, String> writeFn = mock(TableWriteFunction.class);
     doReturn(CompletableFuture.completedFuture(null)).when(writeFn).putAsync(any(), any());
     doReturn(CompletableFuture.completedFuture(null)).when(writeFn).deleteAsync(any());
 
-    final RemoteTable<String, String> remoteTable = new RemoteTable<>(
-        tableId, readFn, writeFn,
-        rateLimitHelper, rateLimitHelper, Executors.newSingleThreadExecutor(),
-        null, null, null,
-        null, null,
-        Executors.newSingleThreadExecutor());
+    final RemoteTable<String, String, String> remoteTable =
+        new RemoteTable<>(tableId, readFn, writeFn, readRateLimitHelper, writeRateLimitHelper, updateRateLimitHelper,
+            Executors.newSingleThreadExecutor(), null, null, null,
+            null, null, Executors.newSingleThreadExecutor());
 
-    final CachingTable<String, String> cachingTable = new CachingTable<>(
+    final CachingTable<String, String, String> cachingTable = new CachingTable<>(
         tableId, remoteTable, guavaTable, false);
 
     initTables(true, cachingTable, guavaTable, remoteTable);
