@@ -26,7 +26,7 @@ import org.apache.samza.metrics.Gauge;
 import org.apache.samza.metrics.MetricsRegistry;
 import org.apache.samza.metrics.Timer;
 import org.apache.samza.storage.kv.Entry;
-import org.apache.samza.table.AsyncReadWriteTable;
+import org.apache.samza.table.AsyncReadWriteUpdateTable;
 import org.apache.samza.table.ratelimit.AsyncRateLimitedTable;
 import org.apache.samza.table.retry.AsyncRetriableTable;
 import org.apache.samza.table.retry.TableRetryPolicy;
@@ -67,16 +67,17 @@ public class TestRemoteTable {
     return context;
   }
 
-  private <K, V, T extends RemoteTable<K, V>> T getTable(String tableId, TableReadFunction<K, V> readFn,
-      TableWriteFunction<K, V> writeFn, boolean retry) {
+  private <K, V, U, T extends RemoteTable<K, V, U>> T getTable(String tableId, TableReadFunction<K, V> readFn,
+      TableWriteFunction<K, V, U> writeFn, boolean retry) {
     return getTable(tableId, readFn, writeFn, null, retry);
   }
 
-  private <K, V, T extends RemoteTable<K, V>> T getTable(String tableId, TableReadFunction<K, V> readFn,
-      TableWriteFunction<K, V> writeFn, ExecutorService cbExecutor, boolean retry) {
+  private <K, V, U, T extends RemoteTable<K, V, U>> T getTable(String tableId, TableReadFunction<K, V> readFn,
+      TableWriteFunction<K, V, U> writeFn, ExecutorService cbExecutor, boolean retry) {
 
     TableRateLimiter<K, V> readRateLimiter = mock(TableRateLimiter.class);
     TableRateLimiter<K, V> writeRateLimiter = mock(TableRateLimiter.class);
+    TableRateLimiter<K, U> updateRateLimiter = mock(TableRateLimiter.class);
 
     TableRetryPolicy readPolicy = retry ? new TableRetryPolicy() : null;
     TableRetryPolicy writePolicy = retry ? new TableRetryPolicy() : null;
@@ -84,8 +85,8 @@ public class TestRemoteTable {
     ExecutorService rateLimitingExecutor = Executors.newSingleThreadExecutor();
     ScheduledExecutorService retryExecutor = Executors.newSingleThreadScheduledExecutor();
 
-    RemoteTable<K, V> table = new RemoteTable(tableId, readFn, writeFn,
-        readRateLimiter, writeRateLimiter, rateLimitingExecutor,
+    RemoteTable<K, V, U> table = new RemoteTable<>(tableId, readFn, writeFn,
+        readRateLimiter, writeRateLimiter, updateRateLimiter, rateLimitingExecutor,
         readPolicy, writePolicy, retryExecutor, null, null, cbExecutor);
     table.init(getMockContext());
     if (readFn != null) {
@@ -119,7 +120,7 @@ public class TestRemoteTable {
     if (retry) {
       doReturn(true).when(readFn).isRetriable(any());
     }
-    RemoteTable<String, String> table = getTable(tableId, readFn, null, retry);
+    RemoteTable<String, String, Void> table = getTable(tableId, readFn, null, retry);
     Assert.assertEquals("bar", sync ? table.get("foo") : table.getAsync("foo").join());
     verify(table.readRateLimiter, times(error && retry ? 2 : 1)).throttle(anyString());
   }
@@ -131,7 +132,7 @@ public class TestRemoteTable {
 
   @Test
   public void testSucceedValidationOnNullReadFn() {
-    RemoteTable<String, String> table = getTable("tableId", null, mock(TableWriteFunction.class), false);
+    RemoteTable<String, String, Void> table = getTable("tableId", null, mock(TableWriteFunction.class), false);
     Assert.assertNotNull(table);
   }
 
@@ -139,10 +140,10 @@ public class TestRemoteTable {
   public void testInit() {
     String tableId = "testInit";
     TableReadFunction<String, String> readFn = mock(TableReadFunction.class);
-    TableWriteFunction<String, String> writeFn = mock(TableWriteFunction.class);
-    RemoteTable<String, String> table = getTable(tableId, readFn, writeFn, true);
+    TableWriteFunction<String, String, String> writeFn = mock(TableWriteFunction.class);
+    RemoteTable<String, String, String> table = getTable(tableId, readFn, writeFn, true);
     // AsyncRetriableTable
-    AsyncReadWriteTable innerTable = TestUtils.getFieldValue(table, "asyncTable");
+    AsyncReadWriteUpdateTable innerTable = TestUtils.getFieldValue(table, "asyncTable");
     Assert.assertTrue(innerTable instanceof AsyncRetriableTable);
     Assert.assertNotNull(TestUtils.getFieldValue(innerTable, "readRetryMetrics"));
     Assert.assertNotNull(TestUtils.getFieldValue(innerTable, "writeRetryMetrics"));
@@ -186,8 +187,8 @@ public class TestRemoteTable {
     doReturn(CompletableFuture.completedFuture("bar1")).when(readFn1).getAsync(anyString());
     doReturn(CompletableFuture.completedFuture("bar2")).when(readFn1).getAsync(anyString());
 
-    RemoteTable<String, String> table1 = getTable("testGetMultipleTables-1", readFn1, null, false);
-    RemoteTable<String, String> table2 = getTable("testGetMultipleTables-2", readFn2, null, false);
+    RemoteTable<String, String, Void> table1 = getTable("testGetMultipleTables-1", readFn1, null, false);
+    RemoteTable<String, String, Void> table2 = getTable("testGetMultipleTables-2", readFn2, null, false);
 
     CompletableFuture<String> future1 = table1.getAsync("foo1");
     CompletableFuture<String> future2 = table2.getAsync("foo2");
@@ -201,7 +202,7 @@ public class TestRemoteTable {
 
   public void doTestRead(boolean sync, boolean error) {
     TableReadFunction<String, String> readFn = mock(TableReadFunction.class);
-    RemoteTable<String, String> table = getTable("testRead-" + sync + error,
+    RemoteTable<String, String, Void> table = getTable("testRead-" + sync + error,
         readFn, mock(TableWriteFunction.class), false);
     CompletableFuture<?> future;
     if (error) {
@@ -238,8 +239,8 @@ public class TestRemoteTable {
 
   private void doTestPut(boolean sync, boolean error, boolean isDelete, boolean retry) {
     String tableId = "testPut-" + sync + error + isDelete + retry;
-    TableWriteFunction<String, String> mockWriteFn = mock(TableWriteFunction.class);
-    TableWriteFunction<String, String> writeFn = mockWriteFn;
+    TableWriteFunction<String, String, String> mockWriteFn = mock(TableWriteFunction.class);
+    TableWriteFunction<String, String, String> writeFn = mockWriteFn;
     CompletableFuture<Void> successFuture = CompletableFuture.completedFuture(null);
     CompletableFuture<Void> failureFuture = new CompletableFuture();
     failureFuture.completeExceptionally(new RuntimeException("Test exception"));
@@ -264,7 +265,7 @@ public class TestRemoteTable {
         doAnswer(args -> times[0]++ == 0 ? failureFuture : successFuture).when(writeFn).putAsync(any(), any());
       }
     }
-    RemoteTable<String, String> table = getTable(tableId, mock(TableReadFunction.class), writeFn, retry);
+    RemoteTable<String, String, String> table = getTable(tableId, mock(TableReadFunction.class), writeFn, retry);
     if (sync) {
       table.put("foo", isDelete ? null : "bar");
     } else {
@@ -316,9 +317,49 @@ public class TestRemoteTable {
     doTestPut(false, true, false, true);
   }
 
+  public void doTestUpdate(boolean sync, boolean error) {
+    String tableId = "testUpdate-" + sync + error;
+    TableWriteFunction<String, String, String> mockWriteFn = mock(TableWriteFunction.class);
+    TableWriteFunction<String, String, String> writeFn = mockWriteFn;
+    CompletableFuture<Void> successFuture = CompletableFuture.completedFuture(null);
+    CompletableFuture<Void> failureFuture = new CompletableFuture();
+    failureFuture.completeExceptionally(new RuntimeException("Test exception"));
+    if (!error) {
+      doReturn(successFuture).when(writeFn).updateAsync(any(), any());
+    } else {
+      doReturn(failureFuture).when(writeFn).updateAsync(any(), any());
+    }
+
+    RemoteTable<String, String, String> table = getTable(tableId, mock(TableReadFunction.class), writeFn, false);
+    if (sync) {
+      table.update("foo", "bar");
+    } else {
+      table.updateAsync("foo", "bar").join();
+    }
+    ArgumentCaptor<String> keyCaptor = ArgumentCaptor.forClass(String.class);
+    ArgumentCaptor<String> updateCaptor = ArgumentCaptor.forClass(String.class);
+    verify(mockWriteFn, times(1))
+        .updateAsync(keyCaptor.capture(), updateCaptor.capture());
+  }
+
+  @Test
+  public void testUpdate() {
+    doTestUpdate(true, false);
+  }
+
+  @Test
+  public void testUpdateAsync() {
+    doTestUpdate(false, false);
+  }
+
+  @Test(expected = RuntimeException.class)
+  public void testUpdateAsyncError() {
+    doTestUpdate(false, true);
+  }
+
   private void doTestDelete(boolean sync, boolean error) {
-    TableWriteFunction<String, String> writeFn = mock(TableWriteFunction.class);
-    RemoteTable<String, String> table = getTable("testDelete-" + sync + error,
+    TableWriteFunction<String, String, String> writeFn = mock(TableWriteFunction.class);
+    RemoteTable<String, String, String> table = getTable("testDelete-" + sync + error,
         mock(TableReadFunction.class), writeFn, false);
     CompletableFuture<Void> future;
     if (error) {
@@ -371,7 +412,7 @@ public class TestRemoteTable {
     }
     // Sync is backed by async so needs to mock the async method
     doReturn(future).when(readFn).getAllAsync(any());
-    RemoteTable<String, String> table = getTable("testGetAll-" + sync + error + partial, readFn, null, false);
+    RemoteTable<String, String, Void> table = getTable("testGetAll-" + sync + error + partial, readFn, null, false);
     Assert.assertEquals(res, sync ? table.getAll(Arrays.asList("foo1", "foo2"))
         : table.getAllAsync(Arrays.asList("foo1", "foo2")).join());
     verify(table.readRateLimiter, times(1)).throttle(anyCollection());
@@ -399,8 +440,8 @@ public class TestRemoteTable {
   }
 
   public void doTestPutAll(boolean sync, boolean error, boolean hasDelete) {
-    TableWriteFunction<String, String> writeFn = mock(TableWriteFunction.class);
-    RemoteTable<String, String> table = getTable("testPutAll-" + sync + error + hasDelete,
+    TableWriteFunction<String, String, Void> writeFn = mock(TableWriteFunction.class);
+    RemoteTable<String, String, Void> table = getTable("testPutAll-" + sync + error + hasDelete,
         mock(TableReadFunction.class), writeFn, false);
     CompletableFuture<Void> future;
     if (error) {
@@ -461,9 +502,49 @@ public class TestRemoteTable {
     doTestPutAll(false, true, false);
   }
 
+  public void doTestUpdateAll(boolean sync, boolean error) {
+    String tableId = "testUpdateAll-" + sync + error;
+    TableWriteFunction<String, String, String> mockWriteFn = mock(TableWriteFunction.class);
+    TableWriteFunction<String, String, String> writeFn = mockWriteFn;
+    CompletableFuture<Void> successFuture = CompletableFuture.completedFuture(null);
+    CompletableFuture<Void> failureFuture = new CompletableFuture();
+    failureFuture.completeExceptionally(new RuntimeException("Test exception"));
+    if (!error) {
+      doReturn(successFuture).when(writeFn).updateAllAsync(anyCollection());
+    } else {
+      doReturn(failureFuture).when(writeFn).updateAllAsync(anyCollection());
+    }
+
+    List<Entry<String, String>> updates = Arrays.asList(new Entry<>("foo1", "bar1"), new Entry<>("foo2", "bar2"));
+
+    RemoteTable<String, String, String> table = getTable(tableId, mock(TableReadFunction.class), writeFn, false);
+    if (sync) {
+      table.updateAll(updates);
+    } else {
+      table.updateAllAsync(updates).join();
+    }
+    ArgumentCaptor<List> argCaptor = ArgumentCaptor.forClass(List.class);
+    verify(mockWriteFn, times(1)).updateAllAsync(argCaptor.capture());
+  }
+
+  @Test
+  public void testUpdateAll() {
+    doTestUpdateAll(true, false);
+  }
+
+  @Test
+  public void testUpdateAllAsync() {
+    doTestUpdateAll(false, false);
+  }
+
+  @Test(expected = RuntimeException.class)
+  public void testUpdateAllAsyncError() {
+    doTestUpdateAll(false, true);
+  }
+
   public void doTestDeleteAll(boolean sync, boolean error) {
-    TableWriteFunction<String, String> writeFn = mock(TableWriteFunction.class);
-    RemoteTable<String, String> table = getTable("testDeleteAll-" + sync + error,
+    TableWriteFunction<String, String, Void> writeFn = mock(TableWriteFunction.class);
+    RemoteTable<String, String, Void> table = getTable("testDeleteAll-" + sync + error,
         mock(TableReadFunction.class), writeFn, false);
     CompletableFuture<Void> future;
     if (error) {
@@ -502,8 +583,8 @@ public class TestRemoteTable {
   }
 
   public void doTestWrite(boolean sync, boolean error) {
-    TableWriteFunction<String, String> writeFn = mock(TableWriteFunction.class);
-    RemoteTable<String, String> table = getTable("testWrite-" + sync + error,
+    TableWriteFunction<String, String, Void> writeFn = mock(TableWriteFunction.class);
+    RemoteTable<String, String, Void> table = getTable("testWrite-" + sync + error,
         mock(TableReadFunction.class), writeFn, false);
     CompletableFuture<?> future;
     if (error) {
@@ -540,8 +621,8 @@ public class TestRemoteTable {
 
   @Test
   public void testFlush() {
-    TableWriteFunction<String, String> writeFn = mock(TableWriteFunction.class);
-    RemoteTable<String, String> table = getTable("testFlush", mock(TableReadFunction.class), writeFn, false);
+    TableWriteFunction<String, String, Void> writeFn = mock(TableWriteFunction.class);
+    RemoteTable<String, String, Void> table = getTable("testFlush", mock(TableReadFunction.class), writeFn, false);
     table.flush();
     verify(writeFn, times(1)).flush();
   }
@@ -551,7 +632,7 @@ public class TestRemoteTable {
     TableReadFunction<String, String> readFn = mock(TableReadFunction.class);
     // Sync is backed by async so needs to mock the async method
     doReturn(CompletableFuture.completedFuture("bar")).when(readFn).getAsync(anyString());
-    RemoteTable<String, String> table = getTable("testGetWithCallbackExecutor", readFn, null,
+    RemoteTable<String, String, Void> table = getTable("testGetWithCallbackExecutor", readFn, null,
         Executors.newSingleThreadExecutor(), false);
     Thread testThread = Thread.currentThread();
 
@@ -573,7 +654,7 @@ public class TestRemoteTable {
     doReturn(CompletableFuture.completedFuture(result)).when(readFn).getAllAsync(any(), any());
     doReturn(CompletableFuture.completedFuture(5)).when(readFn).readAsync(anyInt(), any());
 
-    RemoteTable<String, String> table = getTable("testGetDelegation", readFn, null,
+    RemoteTable<String, String, Void> table = getTable("testGetDelegation", readFn, null,
         Executors.newSingleThreadExecutor(), true);
     verify(readFn, times(1)).init(any(), any());
 
@@ -604,7 +685,7 @@ public class TestRemoteTable {
   }
 
   @Test
-  public void testPutAndDeleteDelegation() {
+  public void testPutUpdateAndDeleteDelegation() {
     TableReadFunction<String, String> readFn = mock(TableReadFunction.class);
     TableWriteFunction writeFn = mock(TableWriteFunction.class);
     doReturn(true).when(writeFn).isRetriable(any());
@@ -612,13 +693,15 @@ public class TestRemoteTable {
     doReturn(CompletableFuture.completedFuture(null)).when(writeFn).putAsync(any(), any(), any());
     doReturn(CompletableFuture.completedFuture(null)).when(writeFn).putAllAsync(anyCollection());
     doReturn(CompletableFuture.completedFuture(null)).when(writeFn).putAllAsync(anyCollection(), any());
+    doReturn(CompletableFuture.completedFuture(null)).when(writeFn).updateAsync(any(), any());
+    doReturn(CompletableFuture.completedFuture(null)).when(writeFn).updateAllAsync(anyCollection());
     doReturn(CompletableFuture.completedFuture(null)).when(writeFn).deleteAsync(any());
     doReturn(CompletableFuture.completedFuture(null)).when(writeFn).deleteAsync(any(), any());
     doReturn(CompletableFuture.completedFuture(null)).when(writeFn).deleteAllAsync(anyCollection());
     doReturn(CompletableFuture.completedFuture(null)).when(writeFn).deleteAllAsync(anyCollection(), any());
     doReturn(CompletableFuture.completedFuture(null)).when(writeFn).writeAsync(anyInt(), any());
 
-    RemoteTable<String, String> table = getTable("testGetDelegation", readFn, writeFn,
+    RemoteTable<String, String, String> table = getTable("testGetDelegation", readFn, writeFn,
         Executors.newSingleThreadExecutor(), true);
     verify(readFn, times(1)).init(any(), any());
 
@@ -640,6 +723,16 @@ public class TestRemoteTable {
     table.putAllAsync(Arrays.asList(new Entry("foo", "bar")), 2).join();
     verify(writeFn, times(1)).putAllAsync(anyCollection());
     verify(writeFn, times(1)).putAllAsync(anyCollection(), any());
+
+    // UpdateAsync
+    verify(writeFn, times(0)).updateAsync(any(), any());
+    table.updateAsync("foo", "bar").join();
+    verify(writeFn, times(1)).updateAsync(any(), any());
+    // UpdateAllAsync
+    verify(writeFn, times(0)).updateAllAsync(anyCollection());
+    table.updateAllAsync(Arrays.asList(new Entry<>("foo", "bar"))).join();
+    verify(writeFn, times(1)).updateAllAsync(anyCollection());
+
     // DeleteAsync
     verify(writeFn, times(0)).deleteAsync(any());
     verify(writeFn, times(0)).deleteAsync(any(), any());
