@@ -21,7 +21,7 @@
 
 package org.apache.samza.util
 
-import org.apache.samza.testUtils.FileUtil
+import org.apache.commons.lang3.RandomStringUtils
 
 import java.io.{File, FileInputStream, FileOutputStream, ObjectInputStream, ObjectOutputStream}
 import org.junit.Assert.{assertEquals, assertNull, assertTrue, fail}
@@ -33,8 +33,9 @@ import scala.util.Random
 class TestFileUtil {
   val data = "100"
   val fileUtil = new FileUtil()
-  val checksum = fileUtil.getChecksum(data)
-  val file = new File(System.getProperty("java.io.tmpdir"), "test")
+  val checksum: Long = fileUtil.getChecksum(data)
+  val tmpDir: String = System.getProperty("java.io.tmpdir")
+  val file = new File(tmpDir, "test")
 
   @Test
   def testWriteDataToFile() {
@@ -53,6 +54,26 @@ class TestFileUtil {
     fis.close()
   }
 
+  @Test
+  def testWriteLargeDataToFile() {
+    val largeData = RandomStringUtils.randomAscii(fileUtil.MaxStringSegmentWriteSize * 2)
+    val largeChecksum = fileUtil.getChecksum(largeData)
+
+    // Invoke test
+    fileUtil.writeWithChecksum(file, largeData)
+
+    // Check that file exists
+    assertTrue("File was not created!", file.exists())
+    val fis = new FileInputStream(file)
+    val ois = new ObjectInputStream(fis)
+
+    // Check content of the file is as expected
+    assertEquals(largeChecksum, ois.readLong())
+    assertEquals(largeData, fileUtil.readWithChecksum(file))
+    ois.close()
+    fis.close()
+  }
+
   @Test
   def testWriteDataToFileWithExistingOffsetFile() {
     // Invoke test
@@ -92,6 +113,18 @@ class TestFileUtil {
     assertEquals(data, result)
   }
 
+  @Test
+  def testReadDataFromBackwardsCompatFile() {
+    // Write
+    fileUtil.writeWithChecksum(file, data)
+
+    // Invoke test
+    val result = fileUtil.readWithChecksum(file)
+
+    // Check data returned
+    assertEquals(data, result)
+  }
+
   @Test
   def testReadInvalidDataFromFile() {
     // Write garbage to produce a null result when it's read
@@ -123,14 +156,14 @@ class TestFileUtil {
      * /tmp/samza-file-util-RANDOM-symlink (symlink to dir above)
      * /tmp/samza-file-util-RANDOM/subdir (created via the symlink above)
      */
-    val tmpDirPath = Paths.get(FileUtil.TMP_DIR)
+    val tmpDirPath = Paths.get(tmpDir)
     val tmpSubDirName = "samza-file-util-" + Random.nextInt()
     val tmpSubDirSymlinkName = tmpSubDirName + "-symlink"
 
-    val tmpSubDirPath = Paths.get(FileUtil.TMP_DIR, tmpSubDirName);
+    val tmpSubDirPath = Paths.get(tmpDir, tmpSubDirName);
     fileUtil.createDirectories(tmpSubDirPath)
 
-    val tmpSymlinkPath = Paths.get(FileUtil.TMP_DIR, tmpSubDirSymlinkName)
+    val tmpSymlinkPath = Paths.get(tmpDir, tmpSubDirSymlinkName)
     Files.createSymbolicLink(tmpSymlinkPath, tmpDirPath);
 
     try {
@@ -146,7 +179,7 @@ class TestFileUtil {
     fileUtil.createDirectories(tmpSymlinkPath)
 
     // verify that subdirs can be created via symlinks correctly.
-    val tmpSubSubDirPath = Paths.get(FileUtil.TMP_DIR, tmpSubDirName + "-symlink", "subdir")
+    val tmpSubSubDirPath = Paths.get(tmpDir, tmpSubDirName + "-symlink", "subdir")
     fileUtil.createDirectories(tmpSubSubDirPath)
   }
 }
