@@ -33,7 +33,7 @@ import org.slf4j.LoggerFactory;
  * Extends StreamSpec with the ability to easily get the topic replication factor.
  */
 public class KafkaStreamSpec extends StreamSpec {
-  private static Logger LOG = LoggerFactory.getLogger(KafkaStreamSpec.class);
+  private static final Logger LOG = LoggerFactory.getLogger(KafkaStreamSpec.class);
 
   private static final int DEFAULT_REPLICATION_FACTOR = 2;
 
@@ -59,10 +59,8 @@ public class KafkaStreamSpec extends StreamSpec {
    * @return            The Map instance.
    */
   private static Map<String, String> propertiesToMap(Properties properties) {
-    Map<String, String> map = new HashMap<String, String>();
-    for (final String name: properties.stringPropertyNames()) {
-      map.put(name, properties.getProperty(name));
-    }
+    Map<String, String> map = new HashMap<>();
+    properties.stringPropertyNames().forEach(name -> map.put(name, properties.getProperty(name)));
     return map;
   }
 
@@ -99,18 +97,18 @@ public class KafkaStreamSpec extends StreamSpec {
    */
   public static KafkaStreamSpec fromSpec(StreamSpec originalSpec) {
     if (originalSpec instanceof KafkaStreamSpec) {
-      return ((KafkaStreamSpec) originalSpec);
+      return (KafkaStreamSpec) originalSpec;
     }
 
-    int replicationFactor = Integer.parseInt(originalSpec.getOrDefault( KafkaConfig.TOPIC_REPLICATION_FACTOR(),
-                                                                        KafkaConfig.TOPIC_DEFAULT_REPLICATION_FACTOR()));
+    int replicationFactor = Integer.parseInt(originalSpec.getOrDefault(KafkaConfig.TOPIC_REPLICATION_FACTOR(),
+        KafkaConfig.TOPIC_DEFAULT_REPLICATION_FACTOR()));
 
-    return new KafkaStreamSpec( originalSpec.getId(),
-                                originalSpec.getPhysicalName(),
-                                originalSpec.getSystemName(),
-                                originalSpec.getPartitionCount(),
-                                replicationFactor,
-                                mapToProperties(filterUnsupportedProperties(originalSpec.getConfig())));
+    return new KafkaStreamSpec(originalSpec.getId(),
+                               originalSpec.getPhysicalName(),
+                               originalSpec.getSystemName(),
+                               originalSpec.getPartitionCount(),
+                               replicationFactor,
+                               mapToProperties(filterUnsupportedProperties(originalSpec.getConfig())));
   }
 
   /**
