@@ -19,6 +19,7 @@
 
 package org.apache.samza.system.kafka;
 
+import java.time.Duration;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Map;
@@ -173,6 +174,11 @@ public class MockKafkaProducer implements Producer<byte[], byte[]> {
     new FlushRunnable(0).run();
   }
 
+  @Override
+  public void close(Duration timeout) {
+    close(timeout.toMillis(), TimeUnit.MILLISECONDS);
+  }
+
   public void open() {
     this.closed = false;
     openCount++;
