@@ -26,7 +26,6 @@ import java.util.Map;
 import java.util.concurrent.BlockingQueue;
 import java.util.concurrent.LinkedBlockingQueue;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicBoolean;
 import org.apache.logging.log4j.core.Filter;
 import org.apache.logging.log4j.core.Layout;
 import org.apache.logging.log4j.core.LogEvent;
@@ -73,7 +72,7 @@ public class StreamAppender extends AbstractAppender {
   private final BlockingQueue<EncodedLogEvent> logQueue = new LinkedBlockingQueue<>(DEFAULT_QUEUE_SIZE);
 
   private SystemStream systemStream = null;
-  private SystemProducer systemProducer = null;
+  protected SystemProducer systemProducer = null;
   private String key = null;
   private byte[] keyBytes; // Serialize the key once, since we will use it for every event.
   private String containerName = null;
@@ -85,12 +84,6 @@ public class StreamAppender extends AbstractAppender {
   private String streamName = null;
   private final boolean usingAsyncLogger;
   private final LoggingContextHolder loggingContextHolder;
-
-  /**
-   * used to detect if this thread is called recursively
-   */
-  private final AtomicBoolean recursiveCall = new AtomicBoolean(false);
-
   protected static final int DEFAULT_QUEUE_SIZE = 100;
   protected volatile boolean systemInitialized = false;
   protected StreamAppenderMetrics metrics;
@@ -189,37 +182,30 @@ public class StreamAppender extends AbstractAppender {
 
   @Override
   public void append(LogEvent event) {
-    if (!recursiveCall.get()) {
-      try {
-        recursiveCall.set(true);
-        if (!systemInitialized) {
-          // configs are needed to set up producer system, so check that before actually initializing
-          if (this.loggingContextHolder.getConfig() != null) {
-            synchronized (this) {
-              if (!systemInitialized) {
-                setupSystem();
-                systemInitialized = true;
-              }
+    try {
+      if (!systemInitialized) {
+        // configs are needed to set up producer system, so check that before actually initializing
+        if (this.loggingContextHolder.getConfig() != null) {
+          synchronized (this) {
+            if (!systemInitialized) {
+              setupSystem();
+              systemInitialized = true;
             }
-            handleEvent(event);
-          } else {
-            // skip sending the log to the stream if initialization can't happen yet
-            System.out.println("Waiting for config to become available before log can be handled");
           }
-        } else {
           handleEvent(event);
+        } else {
+          // skip sending the log to the stream if initialization can't happen yet
+          System.out.println("Waiting for config to become available before log can be handled");
         }
-      } catch (Exception e) {
-        if (metrics != null) { // setupSystem() may not have been invoked yet so metrics can be null here.
-          metrics.logMessagesErrors.inc();
-        }
-        System.err.println(String.format("[%s] Error sending log message:", getName()));
-        e.printStackTrace();
-      } finally {
-        recursiveCall.set(false);
+      } else {
+        handleEvent(event);
       }
-    } else if (metrics != null) { // setupSystem() may not have been invoked yet so metrics can be null here.
-      metrics.recursiveCalls.inc();
+    } catch (Exception e) {
+      if (metrics != null) { // setupSystem() may not have been invoked yet so metrics can be null here.
+        metrics.logMessagesErrors.inc();
+      }
+      System.err.println(String.format("[%s] Error sending log message:", getName()));
+      e.printStackTrace();
     }
   }
 
@@ -416,7 +402,7 @@ public class StreamAppender extends AbstractAppender {
    * Helper method to send a serialized log-event to the systemProducer, and increment respective methods.
    * @param logQueueEntry the serialized log-event to be sent to the systemProducer
    */
-  private void sendEventToSystemProducer(EncodedLogEvent logQueueEntry) {
+  protected void sendEventToSystemProducer(EncodedLogEvent logQueueEntry) {
     metrics.logMessagesBytesSent.inc(logQueueEntry.getEntryValueSize());
     metrics.logMessagesCountSent.inc();
     systemProducer.send(SOURCE, decorateLogEvent(logQueueEntry));
@@ -513,4 +499,4 @@ public class StreamAppender extends AbstractAppender {
       return entryValue.length;
     }
   }
-}
\ No newline at end of file
+}
