@@ -21,11 +21,10 @@ package org.apache.samza.sql.impl;
 
 import org.apache.samza.SamzaException;
 import org.apache.samza.config.Config;
+import org.apache.samza.sql.serializers.SamzaSqlRelMessageSerdeFactory;
+import org.apache.samza.sql.serializers.SamzaSqlRelRecordSerdeFactory;
 import org.apache.samza.table.descriptors.TableDescriptor;
-import org.apache.samza.serializers.JsonSerdeV2;
 import org.apache.samza.serializers.KVSerde;
-import org.apache.samza.sql.data.SamzaSqlCompositeKey;
-import org.apache.samza.sql.data.SamzaSqlRelMessage;
 import org.apache.samza.sql.interfaces.SqlIOResolver;
 import org.apache.samza.sql.interfaces.SqlIOResolverFactory;
 import org.apache.samza.sql.interfaces.SqlIOConfig;
@@ -33,6 +32,9 @@ import org.apache.samza.storage.kv.descriptors.RocksDbTableDescriptor;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.apache.samza.sql.runner.SamzaSqlApplicationConfig.CFG_METADATA_TOPIC_PREFIX;
+import static org.apache.samza.sql.runner.SamzaSqlApplicationConfig.DEFAULT_METADATA_TOPIC_PREFIX;
+
 
 /**
  * Source Resolver implementation that uses static config to return a config corresponding to a system stream.
@@ -52,11 +54,14 @@ public class ConfigBasedIOResolverFactory implements SqlIOResolverFactory {
   }
 
   private class ConfigBasedIOResolver implements SqlIOResolver {
-    private final String SAMZA_SQL_QUERY_TABLE_KEYWORD = "$table";
+    private static final String SAMZA_SQL_QUERY_TABLE_KEYWORD = "$table";
     private final Config config;
+    private final String changeLogStorePrefix;
 
     public ConfigBasedIOResolver(Config config) {
       this.config = config;
+      String metadataTopicPrefix = config.get(CFG_METADATA_TOPIC_PREFIX, DEFAULT_METADATA_TOPIC_PREFIX);
+      this.changeLogStorePrefix = metadataTopicPrefix + (metadataTopicPrefix.isEmpty() ? "" : "_");
     }
 
     @Override
@@ -100,9 +105,13 @@ public class ConfigBasedIOResolverFactory implements SqlIOResolverFactory {
 
       TableDescriptor tableDescriptor = null;
       if (isTable) {
-        tableDescriptor = new RocksDbTableDescriptor("InputTable-" + name, KVSerde.of(
-            new JsonSerdeV2<>(SamzaSqlCompositeKey.class),
-            new JsonSerdeV2<>(SamzaSqlRelMessage.class)));
+        String tableId = changeLogStorePrefix + "InputTable-" + name.replace(".", "-").replace("$", "-");
+        SamzaSqlRelRecordSerdeFactory.SamzaSqlRelRecordSerde keySerde =
+            (SamzaSqlRelRecordSerdeFactory.SamzaSqlRelRecordSerde) new SamzaSqlRelRecordSerdeFactory().getSerde(null, null);
+        SamzaSqlRelMessageSerdeFactory.SamzaSqlRelMessageSerde valueSerde =
+            (SamzaSqlRelMessageSerdeFactory.SamzaSqlRelMessageSerde) new SamzaSqlRelMessageSerdeFactory().getSerde(null, null);
+        tableDescriptor = new RocksDbTableDescriptor(tableId, KVSerde.of(keySerde, valueSerde))
+            .withChangelogEnabled();
       }
 
       return new SqlIOConfig(systemName, streamName, fetchSystemConfigs(systemName), tableDescriptor);
