@@ -19,14 +19,11 @@
 
 package org.apache.samza.sql.translator;
 
-import com.google.common.base.Preconditions;
 import java.util.ArrayList;
-import java.util.HashMap;
+import java.util.Arrays;
 import java.util.List;
-import java.util.Map;
 import java.util.stream.Collectors;
 import java.util.stream.IntStream;
-import org.apache.calcite.DataContext;
 import org.apache.calcite.rel.core.Project;
 import org.apache.calcite.rel.type.RelDataType;
 import org.apache.calcite.rex.RexCall;
@@ -40,9 +37,7 @@ import org.apache.samza.metrics.MetricsRegistry;
 import org.apache.samza.metrics.SamzaHistogram;
 import org.apache.samza.operators.MessageStream;
 import org.apache.samza.operators.functions.MapFunction;
-import org.apache.samza.sql.SamzaSqlRelRecord;
 import org.apache.samza.sql.data.Expression;
-import org.apache.samza.sql.data.SamzaSqlExecutionContext;
 import org.apache.samza.sql.data.SamzaSqlRelMessage;
 import org.apache.samza.sql.data.SamzaSqlRelMsgMetadata;
 import org.apache.samza.sql.runner.SamzaSqlApplicationContext;
@@ -54,7 +49,7 @@ import org.slf4j.LoggerFactory;
  * Translator to translate the Project node in the relational graph to the corresponding StreamGraph
  * implementation.
  */
-public class ProjectTranslator {
+class ProjectTranslator {
 
   private static final Logger LOG = LoggerFactory.getLogger(ProjectTranslator.class);
   //private transient int messageIndex = 0;
@@ -64,122 +59,6 @@ public class ProjectTranslator {
     this.queryId = queryId;
   }
 
-  /**
-   * Converts the resulting row from Calcite Expression Evaluator to SamzaRelRecord to be sent downstream.
-   *
-   * @param objects input objects to be converted
-   * @param rowType Calcite row type of the resulting row
-   * @return return a valid message Stream of type SamzaSqlRelRecord
-   */
-  public static SamzaSqlRelRecord buildSamzaRelRecord(Object[] objects, RelDataType rowType) {
-    Preconditions.checkNotNull(objects, "Input objects can not be null");
-    Preconditions.checkState(rowType.isStruct(), "Row Type has to be a Struct and got " + rowType.getSqlTypeName());
-    Preconditions.checkState(objects.length == rowType.getFieldCount(),
-        "Objects counts and type counts must match " + objects.length + " vs " + rowType.getFieldCount());
-    List<String> names = new ArrayList<>(rowType.getFieldNames());
-    List<Object> values = new ArrayList<>(rowType.getFieldCount());
-    for (int i = 0; i < objects.length; i++) {
-      Object val = objects[i];
-      if (val == null) {
-        values.add(null);
-        continue;
-      }
-      final RelDataType valueType = rowType.getFieldList().get(i).getType();
-      values.add(convertToSamzaSqlType(val, valueType));
-    }
-    return new SamzaSqlRelRecord(names, values);
-  }
-
-  /**
-   * Recursively converts a Primitive Java Object to valid Samza Rel Record field type.
-   *
-   * @param value input value to be converted
-   * @param dataType value type as derived by Calcite
-   * @return SamzaRelRecord or primitive SamzaRelRecord field.
-   *
-   */
-  private static Object convertToSamzaSqlType(Object value, RelDataType dataType) {
-    if (value == null) {
-      return null;
-    }
-    switch (dataType.getSqlTypeName()) {
-      case ROW:
-        List<String> names = new ArrayList<>(dataType.getFieldNames());
-        // Row Struct is represent as Object array in Calcite.
-        Object[] row = (Object[]) value;
-        List<Object> values = new ArrayList<>(row.length);
-        for (int i = 0; i < row.length; i++) {
-          values.add(convertToSamzaSqlType(row[i], dataType.getFieldList().get(i).getType()));
-        }
-        return new SamzaSqlRelRecord(names, values);
-      case MAP:
-        Map<Object, Object> objectMap = (Map<Object, Object>) value;
-        Map<Object, Object> resultMap = new HashMap<>();
-        final RelDataType valuesType = dataType.getValueType();
-        objectMap.forEach((key, v) -> resultMap.put(key, convertToSamzaSqlType(v, valuesType)));
-        return resultMap;
-      case ARRAY:
-        List<Object> objectList = (List<Object>) value;
-        final RelDataType elementsType = dataType.getComponentType();
-        return objectList.stream().map(e -> convertToSamzaSqlType(e, elementsType)).collect(Collectors.toList());
-      case BOOLEAN:
-      case BIGINT:
-      case BINARY:
-      case INTEGER:
-      case TINYINT:
-      case DOUBLE:
-      case FLOAT:
-      case REAL:
-      case VARCHAR:
-      case CHAR:
-      case VARBINARY:
-      case ANY:
-      case OTHER:
-        // today we treat everything else as Type Any or Other, this is not ideal.
-        // this will change when adding timestamps support or more complex non java primitive types.
-        // TODO in a better world we need to add type factory that can do the conversion between calcite and samza.
-        return value;
-      default:
-        // As of today we treat everything else as type ANY
-        throw new IllegalStateException("Unknown SQL type " + dataType.getSqlTypeName());
-    }
-  }
-
-  /**
-   * Converts the Samza Record to a Java Primitive Row format that's in convention with Calcite Enum operators.
-   *
-   * @param samzaSqlRelRecord input record.
-   * @return row of Java Primitive conform to org.apache.calcite.adapter.enumerable.JavaRowFormat#ARRAY
-   */
-  public static Object[] convertToJavaRow(SamzaSqlRelRecord samzaSqlRelRecord) {
-    if (samzaSqlRelRecord == null) {
-      return null;
-    }
-    Object[] inputRow = new Object[samzaSqlRelRecord.getFieldValues().size()];
-    for (int i = 0; i < inputRow.length; i++) {
-      inputRow[i] = asPrimitiveJavaRow(samzaSqlRelRecord.getFieldValues().get(i));
-    }
-    return inputRow;
-  }
-
-  private static Object asPrimitiveJavaRow(Object inputObject) {
-    if (inputObject == null) {
-      return null;
-    }
-    if (inputObject instanceof SamzaSqlRelRecord) {
-      return convertToJavaRow((SamzaSqlRelRecord) inputObject);
-    }
-    if (inputObject instanceof List) {
-      return ((List) inputObject).stream().map(e -> asPrimitiveJavaRow(e)).collect(Collectors.toList());
-    }
-    if (inputObject instanceof Map) {
-      Map<Object, Object> objectMap = new HashMap<>();
-      ((Map<Object, Object>) inputObject).forEach((k, v) -> objectMap.put(k, asPrimitiveJavaRow(v)));
-      return objectMap;
-    }
-    return inputObject;
-  }
-
   /**
    * ProjectMapFunction implements MapFunction to map input SamzaSqlRelMessages, one at a time, to a new
    * SamzaSqlRelMessage which consists of the projected fields
@@ -214,7 +93,6 @@ public class ProjectTranslator {
       this.translatorContext =
           ((SamzaSqlApplicationContext) context.getApplicationTaskContext()).getTranslatorContexts().get(queryId);
       this.project = (Project) this.translatorContext.getRelNode(projectId);
-      LOG.info("Compiling operator {} ", project.getDigest());
       this.expr = this.translatorContext.getExpressionCompiler().compile(project.getInputs(), project.getProjects());
       ContainerContext containerContext = context.getContainerContext();
       metricsRegistry = containerContext.getContainerMetricsRegistry();
@@ -235,19 +113,20 @@ public class ProjectTranslator {
       long arrivalTime = System.nanoTime();
       RelDataType type = project.getRowType();
       Object[] output = new Object[type.getFieldCount()];
-      Object[] inputRow = convertToJavaRow(message.getSamzaSqlRelRecord());
-      SamzaSqlExecutionContext execContext = translatorContext.getExecutionContext();
-      DataContext dataRootContext = translatorContext.getDataContext();
       try {
-        expr.execute(execContext, context, dataRootContext, inputRow, output);
+        expr.execute(translatorContext.getExecutionContext(), context, translatorContext.getDataContext(),
+            message.getSamzaSqlRelRecord().getFieldValues().toArray(), output);
       } catch (Exception e) {
         String errMsg = String.format("Handling the following rel message ran into an error. %s", message);
         LOG.error(errMsg, e);
         throw new SamzaException(errMsg, e);
       }
-      SamzaSqlRelRecord record = buildSamzaRelRecord(output, project.getRowType());
+      List<String> names = new ArrayList<>();
+      for (int index = 0; index < output.length; index++) {
+        names.add(index, project.getNamedProjects().get(index).getValue());
+      }
       updateMetrics(arrivalTime, System.nanoTime(), message.getSamzaSqlRelMsgMetadata().isNewInputMessage);
-      return new SamzaSqlRelMessage(record, message.getSamzaSqlRelMsgMetadata());
+      return new SamzaSqlRelMessage(names, Arrays.asList(output), message.getSamzaSqlRelMsgMetadata());
     }
 
     /**
