@@ -18,7 +18,6 @@
 */
 package org.apache.samza.sql.translator;
 
-import com.google.common.collect.ImmutableList;
 import java.io.IOException;
 import java.util.ArrayList;
 import java.util.Collections;
@@ -29,10 +28,7 @@ import org.apache.calcite.DataContext;
 import org.apache.calcite.rel.RelNode;
 import org.apache.calcite.rel.logical.LogicalProject;
 import org.apache.calcite.rel.type.RelDataType;
-import org.apache.calcite.rel.type.RelDataTypeField;
-import org.apache.calcite.rel.type.RelDataTypeFieldImpl;
 import org.apache.calcite.rex.RexNode;
-import org.apache.calcite.sql.type.SqlTypeName;
 import org.apache.calcite.util.Pair;
 import org.apache.samza.application.descriptors.StreamApplicationDescriptorImpl;
 import org.apache.samza.context.ContainerContext;
@@ -72,7 +68,6 @@ import static org.mockito.Mockito.when;
 @PrepareForTest(LogicalProject.class)
 public class TestProjectTranslator extends TranslatorTestBase {
   private static final String LOGICAL_OP_ID = "sql0_project_0";
-  private static final String TEST_FIELD = "test_field";
 
   @Test
   public void testTranslate() throws IOException, ClassNotFoundException {
@@ -91,23 +86,12 @@ public class TestProjectTranslator extends TranslatorTestBase {
     when(mockProject.getInputs()).thenReturn(inputs);
     when(mockProject.getInput()).thenReturn(mockInput);
     RelDataType mockRowType = mock(RelDataType.class);
-    List<RelDataTypeField> relFields = new ArrayList<>();
-    String fieldName = TEST_FIELD;
-    int fieldPos = 0;
-    RelDataType dataType = mock(RelDataType.class);
-    when(dataType.getSqlTypeName()).thenReturn(SqlTypeName.ANY);
-    relFields.add(new RelDataTypeFieldImpl(fieldName, fieldPos, dataType));
     when(mockRowType.getFieldCount()).thenReturn(1);
     when(mockProject.getRowType()).thenReturn(mockRowType);
-    when(mockProject.getRowType().getSqlTypeName()).thenReturn(SqlTypeName.ROW);
-    when(mockProject.getRowType().getFieldList()).thenReturn(relFields);
-    when(mockProject.getRowType().isStruct()).thenReturn(true);
     RexNode mockRexField = mock(RexNode.class);
     List<Pair<RexNode, String>> namedProjects = new ArrayList<>();
-    namedProjects.add(Pair.of(mockRexField, TEST_FIELD));
+    namedProjects.add(Pair.of(mockRexField, "test_field"));
     when(mockProject.getNamedProjects()).thenReturn(namedProjects);
-    when(mockProject.getRowType()).thenReturn(mockRowType);
-    when(mockProject.getRowType().getFieldNames()).thenReturn(ImmutableList.of(TEST_FIELD));
     StreamApplicationDescriptorImpl mockAppDesc = mock(StreamApplicationDescriptorImpl.class);
     OperatorSpec<Object, SamzaSqlRelMessage> mockInputOp = mock(OperatorSpec.class);
     MessageStream<SamzaSqlRelMessage> mockStream = new MessageStreamImpl<>(mockAppDesc, mockInputOp);
@@ -168,7 +152,7 @@ public class TestProjectTranslator extends TranslatorTestBase {
     }).when(mockExpr).execute(eq(executionContext), eq(mockContext), eq(dataContext),
         eq(mockInputMsg.getSamzaSqlRelRecord().getFieldValues().toArray()), eq(result));
     SamzaSqlRelMessage retMsg = (SamzaSqlRelMessage) mapFn.apply(mockInputMsg);
-    assertEquals(retMsg.getSamzaSqlRelRecord().getFieldNames(), Collections.singletonList(TEST_FIELD));
+    assertEquals(retMsg.getSamzaSqlRelRecord().getFieldNames(), Collections.singletonList("test_field"));
     assertEquals(retMsg.getSamzaSqlRelRecord().getFieldValues(), Collections.singletonList(mockFieldObj));
 
     // Verify mapFn.apply() updates the TestMetricsRegistryImpl metrics
