@@ -19,7 +19,6 @@
 package org.apache.samza.test.operator;
 
 import java.util.Arrays;
-import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -92,14 +91,14 @@ public class TestRepartitionJoinWindowApp extends StreamApplicationIntegrationTe
     runApplication(app, appName, configs);
 
     // consume and validate result
-    List<ConsumerRecord<String, String>> messages = consumeMessages(Collections.singletonList(outputTopicName), 2);
+    List<ConsumerRecord<String, String>> messages = consumeMessages(outputTopicName, 2);
     assertEquals(2, messages.size());
 
     Assert.assertFalse(KafkaSystemAdmin.deleteMessageCalled);
   }
 
   @Test
-  public void testRepartitionJoinWindowAppAndDeleteMessagesOnCommit() throws Exception {
+  public void ittestRepartitionJoinWindowAppAndDeleteMessagesOnCommit() throws Exception {
     String inputTopicName1 = "page-views2";
     String inputTopicName2 = "ad-clicks2";
     String outputTopicName = "user-ad-click-counts2";
@@ -121,7 +120,7 @@ public class TestRepartitionJoinWindowApp extends StreamApplicationIntegrationTe
     runApplication(app, appName, configs);
 
     // consume and validate result
-    List<ConsumerRecord<String, String>> messages = consumeMessages(Collections.singletonList(outputTopicName), 2);
+    List<ConsumerRecord<String, String>> messages = consumeMessages(outputTopicName, 2);
     assertEquals(2, messages.size());
 
     for (ConsumerRecord<String, String> message : messages) {
