@@ -19,35 +19,55 @@
 
 package org.apache.samza.test.operator;
 
+import java.time.Duration;
+import org.apache.samza.application.descriptors.StreamApplicationDescriptor;
 import org.apache.samza.application.StreamApplication;
 import org.apache.samza.config.Config;
 import org.apache.samza.operators.KV;
 import org.apache.samza.operators.MessageStream;
 import org.apache.samza.operators.OutputStream;
-import org.apache.samza.operators.StreamGraph;
 import org.apache.samza.operators.windows.Windows;
+import org.apache.samza.runtime.ApplicationRunner;
+import org.apache.samza.runtime.ApplicationRunners;
 import org.apache.samza.serializers.IntegerSerde;
 import org.apache.samza.serializers.JsonSerdeV2;
 import org.apache.samza.serializers.KVSerde;
 import org.apache.samza.serializers.StringSerde;
+import org.apache.samza.system.kafka.descriptors.KafkaInputDescriptor;
+import org.apache.samza.system.kafka.descriptors.KafkaOutputDescriptor;
+import org.apache.samza.system.kafka.descriptors.KafkaSystemDescriptor;
 import org.apache.samza.test.operator.data.PageView;
+import org.apache.samza.util.CommandLine;
 
-import java.time.Duration;
 
 /**
  * A {@link StreamApplication} that demonstrates a filter followed by a tumbling window.
  */
 public class TumblingWindowApp implements StreamApplication {
+  private static final String SYSTEM = "kafka";
   private static final String INPUT_TOPIC = "page-views";
   private static final String OUTPUT_TOPIC = "page-view-counts";
   private static final String FILTER_KEY = "badKey";
 
+  public static void main(String[] args) {
+    CommandLine cmdLine = new CommandLine();
+    Config config = cmdLine.loadConfig(cmdLine.parser().parse(args));
+    ApplicationRunner runner = ApplicationRunners.getApplicationRunner(new TumblingWindowApp(), config);
+
+    runner.run();
+    runner.waitForFinish();
+  }
+
   @Override
-  public void init(StreamGraph graph, Config config) {
-    MessageStream<PageView> pageViews =
-        graph.getInputStream(INPUT_TOPIC, new JsonSerdeV2<>(PageView.class));
-    OutputStream<KV<String, Integer>> outputStream =
-        graph.getOutputStream(OUTPUT_TOPIC, new KVSerde<>(new StringSerde(), new IntegerSerde()));
+  public void describe(StreamApplicationDescriptor appDescriptor) {
+    JsonSerdeV2<PageView> inputSerde = new JsonSerdeV2<>(PageView.class);
+    KVSerde<String, Integer> outputSerde = KVSerde.of(new StringSerde(), new IntegerSerde());
+    KafkaSystemDescriptor ksd = new KafkaSystemDescriptor(SYSTEM);
+    KafkaInputDescriptor<PageView> id = ksd.getInputDescriptor(INPUT_TOPIC, inputSerde);
+    KafkaOutputDescriptor<KV<String, Integer>> od = ksd.getOutputDescriptor(OUTPUT_TOPIC, outputSerde);
+
+    MessageStream<PageView> pageViews = appDescriptor.getInputStream(id);
+    OutputStream<KV<String, Integer>> outputStream = appDescriptor.getOutputStream(od);
 
     pageViews
         .filter(m -> !FILTER_KEY.equals(m.getUserId()))
@@ -55,5 +75,6 @@ public class TumblingWindowApp implements StreamApplication {
             new StringSerde(), new JsonSerdeV2<>(PageView.class)), "tumblingWindow")
         .map(m -> KV.of(m.getKey().getKey(), m.getMessage().size()))
         .sendTo(outputStream);
+
   }
 }
