@@ -515,6 +515,7 @@ public class TestSamzaSqlEndToEnd extends SamzaSqlIntegrationTestHarness {
     Assert.assertEquals(IntStream.range(0, numMessages).boxed().collect(Collectors.toList()), outMessages);
   }
 
+  @Ignore
   @Test
   public void testEndToEndNestedRecord() throws SamzaSqlValidatorException {
     int numMessages = 10;
@@ -522,12 +523,9 @@ public class TestSamzaSqlEndToEnd extends SamzaSqlIntegrationTestHarness {
     Map<String, String> staticConfigs = SamzaSqlTestConfig.fetchStaticConfigsWithFactories(numMessages);
 
     String sql1 =
-        "Insert into testavro.outputTopic (id, bool_value)"
-            // SQL array is one indexed.
-            + " select `phoneNumbers`[1].`kind` as string_value, p.address.streetnum.number as id, "
-            + " `phoneNumbers`[1].`kind` = 'Home' as bool_value, cast(p.address.zip as bigint) as long_value"
-            + " from testavro.PROFILE as p where p.address.zip > 0 and p.address.zip < 100003 ";
-
+        "Insert into testavro.outputTopic"
+            + " select `phoneNumbers`[0].`kind`"
+            + " from testavro.PROFILE as p";
     List<String> sqlStmts = Collections.singletonList(sql1);
     staticConfigs.put(SamzaSqlApplicationConfig.CFG_SQL_STMTS_JSON, JsonUtil.toJson(sqlStmts));
 
@@ -541,72 +539,6 @@ public class TestSamzaSqlEndToEnd extends SamzaSqlIntegrationTestHarness {
     Assert.assertEquals(numMessages, outMessages.size());
   }
 
-  /**
-   * Testing the getNestedField built in operator
-   * @throws SamzaSqlValidatorException
-   */
-  @Test
-  public void testEndToEndGetNestedFieldOperator() throws SamzaSqlValidatorException {
-    int numMessages = 10;
-    TestAvroSystemFactory.messages.clear();
-    Map<String, String> staticConfigs = SamzaSqlTestConfig.fetchStaticConfigsWithFactories(numMessages);
-    String sql1 =
-        "Insert into testavro.outputTopic (string_value, id, bool_value, double_value, map_values, long_value)"
-            + " select GetNestedField(address, 'streetnum.number') * getNestedField(mapValues['key'], 'id') as id, "
-            + " cast(GetNestedField(address, 'streetnum').number * 1.0 as double) as double_value, mapValues as map_values, "
-            + " GetNestedField(phoneNumbers[1] ,'kind') = 'Home' as bool_value, cast( mapValues['key'].id as bigint) as long_value , "
-            + " GetNestedField(mapValues['key'], 'name') as string_value "
-            + " from testavro.PROFILE as p  where GetNestedField(address, 'zip') > 0 and GetNestedField(address, 'zip') < 100003";
-    List<String> sqlStmts = Collections.singletonList(sql1);
-    staticConfigs.put(SamzaSqlApplicationConfig.CFG_SQL_STMTS_JSON, JsonUtil.toJson(sqlStmts));
-
-    Config config = new MapConfig(staticConfigs);
-    new SamzaSqlValidator(config).validate(sqlStmts);
-    runApplication(config);
-
-    List<OutgoingMessageEnvelope> outMessages = new ArrayList<>(TestAvroSystemFactory.messages);
-    // check that the projected values are not null, correct types and good values when easy to check.
-    List<GenericRecord> actualResult = outMessages.stream()
-        .map(x -> (GenericRecord) x.getMessage())
-        .filter(x -> (Boolean) x.get("bool_value"))
-        .filter(x -> x.get("string_value") != null && !x.get("string_value").toString().isEmpty())
-        .filter(x -> x.get("map_values") instanceof Map)
-        .filter(x -> x.get("id") instanceof Integer)
-        .filter(x -> (Long) x.get("long_value") < 10 && (Long) x.get("long_value") >= 0)
-        .filter(x -> x.get("double_value") instanceof Double && (Double) x.get("double_value") >= 1234.0)
-        .collect(Collectors.toList());
-    Assert.assertEquals(
-        "Wrong results size, check the test condition against the Actual outputs -> " + outMessages.toString(),
-        numMessages, actualResult.size());
-  }
-
-
-  @Test
-  public void testEndToEndNestedRecordProjectFilter() throws SamzaSqlValidatorException {
-    int numMessages = 10;
-    TestAvroSystemFactory.messages.clear();
-    Map<String, String> staticConfigs = SamzaSqlTestConfig.fetchStaticConfigsWithFactories(numMessages);
-
-    String sql1 = " Insert into testavro.PROFILE1 select (p.address.streetnum.number * p.address.zip) as id , "
-        + " p.address, `phoneNumbers`[1].`kind` = 'Home' as selfEmployed, "
-        + " MAP[cast(id as varchar), `phoneNumbers`[1].number] as mapValues, phoneNumbers, "
-        + " cast(companyId as varchar) || name ||`phoneNumbers`[1].number || 'concat' as name , "
-        + " 100 * ((companyId + 122) / 3 ) as companyId "
-        + " from testavro.PROFILE as p where p.address.zip > 0 "
-        + " and p.address.zip < 100003 and p.address.streetnum.number > 0 ";
-
-    List<String> sqlStmts = Collections.singletonList(sql1);
-    staticConfigs.put(SamzaSqlApplicationConfig.CFG_SQL_STMTS_JSON, JsonUtil.toJson(sqlStmts));
-
-    Config config = new MapConfig(staticConfigs);
-    new SamzaSqlValidator(config).validate(sqlStmts);
-
-    runApplication(config);
-
-    List<OutgoingMessageEnvelope> outMessages = new ArrayList<>(TestAvroSystemFactory.messages);
-    Assert.assertEquals(numMessages, outMessages.size());
-  }
-
   @Test
   public void testEndToEndFlattenWithUdf() throws Exception {
     int numMessages = 20;
@@ -769,7 +701,7 @@ public class TestSamzaSqlEndToEnd extends SamzaSqlIntegrationTestHarness {
     TestAvroSystemFactory.messages.clear();
     Map<String, String> staticConfigs = SamzaSqlTestConfig.fetchStaticConfigsWithFactories(numMessages);
     String sql1 = "Insert into testavro.PROFILE1(id, address) "
-        + "select id, BuildOutputRecord('key', p.address.zip) as address from testavro.PROFILE as p";
+        + "select id, BuildOutputRecord('key', GetNestedField(address, 'zip')) as address from testavro.PROFILE";
     List<String> sqlStmts = Collections.singletonList(sql1);
     staticConfigs.put(SamzaSqlApplicationConfig.CFG_SQL_STMTS_JSON, JsonUtil.toJson(sqlStmts));
     runApplication(new MapConfig(staticConfigs));
@@ -846,7 +778,7 @@ public class TestSamzaSqlEndToEnd extends SamzaSqlIntegrationTestHarness {
             + "       p.name as profileName, p.address as profileAddress "
             + "from testavro.PROFILE.`$table` as p "
             + "join testavro.PAGEVIEW as pv "
-            + " on p.id = pv.profileId where p.name = 'Mike' or p.name is not null";
+            + " on p.id = pv.profileId";
 
     List<String> sqlStmts = Arrays.asList(sql);
     staticConfigs.put(SamzaSqlApplicationConfig.CFG_SQL_STMTS_JSON, JsonUtil.toJson(sqlStmts));
@@ -867,7 +799,7 @@ public class TestSamzaSqlEndToEnd extends SamzaSqlIntegrationTestHarness {
   }
 
   @Test
-  public void testEndToEndStreamTableInnerJoinWithPrimaryKey() {
+  public void testEndToEndStreamTableInnerJoinWithPrimaryKey() throws Exception {
     int numMessages = 20;
 
     TestAvroSystemFactory.messages.clear();
